@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2000, 2012, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -252,7 +252,7 @@ mysql_list_dbcolumns(STMT *stmt,
   @param[in] cbColumn         Length of column pattern
 */
 SQLRETURN
-mysql_columns(STMT *stmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
+mysql_columns(STMT * stmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
              SQLCHAR *szSchema __attribute__((unused)),
              SQLSMALLINT cbSchema __attribute__((unused)),
              SQLCHAR *szTable, SQLSMALLINT cbTable,
@@ -292,7 +292,7 @@ mysql_columns(STMT *stmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
   stmt->result= res;
   alloc= &res->field_alloc;
 
-  if (!(stmt->dbc->flag & FLAG_NO_CATALOG))
+  if (!stmt->dbc->ds->no_catalog)
     db= strmake_root(alloc, (char *)szCatalog, cbCatalog);
 
   while ((table_row= mysql_fetch_row(res)))
@@ -357,7 +357,7 @@ mysql_columns(STMT *stmt, SQLCHAR *szCatalog, SQLSMALLINT cbCatalog,
       }
 
       /* COLUMN_SIZE */
-      fill_column_size_buff(buff, stmt, field, FALSE);
+      fill_column_size_buff(buff, stmt, field);
       row[6]= strdup_root(alloc, buff);
 
       /* BUFFER_LENGTH */
@@ -1021,9 +1021,8 @@ SQLRETURN mysql_foreign_keys(SQLHSTMT hstmt,
               /* FKTABLE_CAT */
               data[4]= (szFkCatalogName ?
                         strdup_root(alloc, (char *)szFkCatalogName) :
-                        (stmt->dbc->database ?
-                          strdup_root(alloc, stmt->dbc->database) :
-                          "null"));
+                        strdup_root(alloc, stmt->dbc->database ?
+                          stmt->dbc->database : "null"));
               data[5]= NULL;                         /* FKTABLE_SCHEM */
               data[6]= row[0];                       /* FKTABLE_TABLE */
 
@@ -1766,7 +1765,7 @@ mysql_special_columns(SQLHSTMT hstmt, SQLUSMALLINT fColType,
             row[3]= strdup_root(alloc,buff);
             sprintf(buff,"%d",type);
             row[2]= strdup_root(alloc,buff);
-            fill_column_size_buff(buff, stmt, field, FALSE);
+            fill_column_size_buff(buff, stmt, field);
             row[4]= strdup_root(alloc,buff);
             sprintf(buff, "%ld", get_transfer_octet_length(stmt, field));
             row[5]= strdup_root(alloc,buff);
@@ -1842,7 +1841,7 @@ mysql_special_columns(SQLHSTMT hstmt, SQLUSMALLINT fColType,
         row[3]= strdup_root(alloc,buff);
         sprintf(buff,"%d",type);
         row[2]= strdup_root(alloc,buff);
-        fill_column_size_buff(buff, stmt, field, FALSE);
+        fill_column_size_buff(buff, stmt, field);
         row[4]= strdup_root(alloc,buff);
         sprintf(buff,"%ld", get_transfer_octet_length(stmt, field));
         row[5]= strdup_root(alloc,buff);
@@ -1939,7 +1938,7 @@ mysql_statistics(SQLHSTMT hstmt,
       return handle_connection_error(stmt);
     }
 
-    if (stmt->dbc->flag & FLAG_NO_CATALOG)
+    if (stmt->dbc->ds->no_catalog)
       stmt->array[0]= "";
     else
       stmt->array[0]= strmake_root(&stmt->result->field_alloc,
@@ -2149,7 +2148,7 @@ mysql_tables(SQLHSTMT hstmt,
 
       data= stmt->result_array;
 
-      if (!(stmt->dbc->flag & FLAG_NO_CATALOG))
+      if (!stmt->dbc->ds->no_catalog)
       {
         if (!catalog)
         {
@@ -2173,14 +2172,19 @@ mysql_tables(SQLHSTMT hstmt,
 
       while ((row= mysql_fetch_row(stmt->result)))
       {
+        int cat_index= 3;
         int type_index= 2;
         int comment_index= 1;
         my_bool view;
 
-        /* TODO Check this condition is really needed here */
-        if (no_information_schema(stmt)
+        /* If if did not use I_S */
+        if (stmt->dbc->ds->no_information_schema
           || !server_has_i_s(stmt->dbc))
+        {
           type_index= comment_index= (stmt->result->field_count == 18) ? 17 : 15;
+          /* We do not have catalog in result */
+          cat_index= -1;
+        }
 
         view= (myodbc_casecmp(row[type_index], "VIEW", 4) == 0);
 
@@ -2190,7 +2194,10 @@ mysql_tables(SQLHSTMT hstmt,
           continue;
         }
 
-        data[0]= db;
+        /*TODO solve no_i_s case for SQL_ALL_CATALOGS */
+        data[0]= (cat_index >= 0 ?
+                  strdup_root(&stmt->result->field_alloc, row[cat_index]) :
+                  db);
         data[1]= "";
         data[2]= strdup_root(&stmt->result->field_alloc, row[0]);
         data[3]= view ? "VIEW" : "TABLE";
