@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -28,18 +28,9 @@
 */
 
 #include "driver.h"
-
-#ifndef _UNIX_
-# ifdef USE_IODBC
-#  include <iodbcinst.h>
-# else
-#  include <odbcinst.h>
-# endif
-#endif
-
-#include "MYODBCUtil.h"
 #include "installer.h"
 #include "stringutil.h"
+#include "MYODBCUtil.h"
 
 #ifndef CLIENT_NO_SCHEMA
 # define CLIENT_NO_SCHEMA      16
@@ -48,99 +39,6 @@
 typedef BOOL (*PromptFunc)(SQLHWND, SQLWCHAR *, SQLUSMALLINT,
                            SQLWCHAR *, SQLSMALLINT, SQLSMALLINT *);
 
-const char *my_os_charset_to_mysql_charset(const char *csname);
-
-
-void ds5_to_ds3(DataSource *ds5, MYODBCUTIL_DATASOURCE *ds, CHARSET_INFO  *clientcs)
-{
-  SQLUINTEGER token_len= SQL_NTS;
-  uint        errors;
-
-  if (ds5->name)
-  {
-    ds->pszDSN=         sqlwchar_as_sqlchar(clientcs, ds5->name, &token_len, &errors);
-    token_len= SQL_NTS;
-  }
-  if (ds5->description)
-  {
-    ds->pszDESCRIPTION= sqlwchar_as_sqlchar(clientcs, ds5->description, &token_len, &errors);
-    token_len= SQL_NTS;
-  }
-  if (ds5->server)
-  {
-    ds->pszSERVER=      sqlwchar_as_sqlchar(clientcs, ds5->server, &token_len, &errors);
-	  token_len= SQL_NTS;
-  }
-  if (ds5->uid)
-  {
-    ds->pszUSER=        sqlwchar_as_sqlchar(clientcs, ds5->uid, &token_len, &errors);
-    token_len= SQL_NTS;
-  }
-  if (ds5->pwd)
-  {
-    ds->pszPASSWORD=    sqlwchar_as_sqlchar(clientcs, ds5->pwd, &token_len, &errors);
-    token_len= SQL_NTS;
-  }
-  if (ds5->database)
-  {
-    ds->pszDATABASE=    sqlwchar_as_sqlchar(clientcs, ds5->database, &token_len, &errors);
-    token_len= SQL_NTS;
-  }
-  if (ds5->socket)
-  {
-    ds->pszSOCKET=      sqlwchar_as_sqlchar(clientcs, ds5->socket, &token_len, &errors);
-    token_len= SQL_NTS;
-  }
-  if (ds5->initstmt)
-  {
-    ds->pszSTMT=        sqlwchar_as_sqlchar(clientcs, ds5->initstmt, &token_len, &errors);
-    token_len= SQL_NTS;
-  }
-  if (ds_get_options(ds5))
-  {
-    ds->pszOPTION=      _global_alloc(20);
-    sprintf(ds->pszOPTION, "%u", ds_get_options(ds5));
-  }
-  ds->bINTERACTIVE=    (ds5->clientinteractive != 0);
-
-  if (ds5->sslkey)
-  {
-    ds->pszSSLKEY=      sqlwchar_as_sqlchar(clientcs, ds5->sslkey, &token_len, &errors);
-    token_len= SQL_NTS;
-  }
-  if (ds5->sslcert)
-  {
-    ds->pszSSLCERT=     sqlwchar_as_sqlchar(clientcs, ds5->sslcert, &token_len, &errors);
-    token_len= SQL_NTS;
-  }
-  if (ds5->sslca)
-  {
-    ds->pszSSLCA=       sqlwchar_as_sqlchar(clientcs, ds5->sslca, &token_len, &errors);
-    token_len= SQL_NTS;
-  }
-  if (ds5->sslcapath)
-  {
-    ds->pszSSLCAPATH=   sqlwchar_as_sqlchar(clientcs, ds5->sslcapath, &token_len, &errors);
-    token_len= SQL_NTS;
-  }
-  if (ds5->sslcipher)
-  {
-    ds->pszSSLCIPHER=   sqlwchar_as_sqlchar(clientcs, ds5->sslcipher, &token_len, &errors);
-    token_len= SQL_NTS;
-  }
-  if (ds5->charset)
-  {
-    ds->pszCHARSET=     sqlwchar_as_sqlchar(clientcs, ds5->charset, &token_len, &errors);
-    token_len= SQL_NTS;
-  }
-
-  ds->pszPORT= _global_strdup("        ");
-  sprintf(ds->pszPORT, "%d", ds5->port);
-
-  ds->pszSSLVERIFY= _global_strdup(" ");
-  sprintf(ds->pszSSLVERIFY,"%d", ds5->sslverify);
-}
-
 /**
   Get the connection flags based on the driver options.
 
@@ -148,29 +46,27 @@ void ds5_to_ds3(DataSource *ds5, MYODBCUTIL_DATASOURCE *ds, CHARSET_INFO  *clien
 
   @return Client flags suitable for @c mysql_real_connect().
 */
-unsigned long get_client_flags(unsigned long options)
+unsigned long get_client_flags(DataSource *ds)
 {
   unsigned long flags= CLIENT_MULTI_RESULTS;
 
-  if (options & (FLAG_FOUND_ROWS | FLAG_SAFE))
+  if (ds->safe || ds->return_matching_rows)
     flags|= CLIENT_FOUND_ROWS;
-  if (options & FLAG_NO_SCHEMA)
+  if (ds->no_catalog)
     flags|= CLIENT_NO_SCHEMA;
-  if (options & FLAG_COMPRESSED_PROTO)
+  if (ds->use_compressed_protocol)
     flags|= CLIENT_COMPRESS;
-  if (options & FLAG_IGNORE_SPACE)
+  if (ds->ignore_space_after_function_names)
     flags|= CLIENT_IGNORE_SPACE;
-  if (options & FLAG_MULTI_STATEMENTS)
+  if (ds->allow_multiple_statements)
     flags|= CLIENT_MULTI_STATEMENTS;
+  if (ds->clientinteractive)
+    flags|= CLIENT_INTERACTIVE;
 
   return flags;
 }
 
 
-/* Function from mysqlclient */
-int mysql_init_character_set(MYSQL *mysql);
-
-
 /**
  If it was specified, set the character set for the connection.
 
@@ -179,12 +75,56 @@ int mysql_init_character_set(MYSQL *mysql);
 */
 SQLRETURN myodbc_set_initial_character_set(DBC *dbc, const char *charset)
 {
-#if ((MYSQL_VERSION_ID >= 40113 && MYSQL_VERSION_ID < 50000) || \
-     MYSQL_VERSION_ID >= 50006)
-  if (charset && charset[0] && mysql_set_character_set(&dbc->mysql, charset))
-    return set_dbc_error(dbc, "HY000", mysql_error(&dbc->mysql),
-                         mysql_errno(&dbc->mysql));
-#endif
+  if (dbc->unicode)
+  {
+    if (charset && charset[0])
+    {
+      dbc->ansi_charset_info= get_charset_by_csname(charset,
+                                                    MYF(MY_CS_PRIMARY),
+                                                    MYF(0));
+    }
+
+    charset= "utf8";
+  }
+
+  if (charset && charset[0])
+  {
+    if (mysql_set_character_set(&dbc->mysql, charset))
+    {
+      set_dbc_error(dbc, "HY000", mysql_error(&dbc->mysql),
+                    mysql_errno(&dbc->mysql));
+      return SQL_ERROR;
+    }
+  }
+  else
+  {
+    if (mysql_set_character_set(&dbc->mysql, dbc->ansi_charset_info->csname))
+    {
+      set_dbc_error(dbc, "HY000", mysql_error(&dbc->mysql),
+                    mysql_errno(&dbc->mysql));
+      return SQL_ERROR;
+    }
+  }
+
+  {
+    MY_CHARSET_INFO my_charset;
+    mysql_get_character_set_info(&dbc->mysql, &my_charset);
+    dbc->cxn_charset_info= get_charset(my_charset.number, MYF(0));
+  }
+
+  if (!dbc->unicode)
+    dbc->ansi_charset_info= dbc->cxn_charset_info;
+
+  /*
+    We always set character_set_results to NULL so we can do our own
+    conversion to the ANSI character set or Unicode.
+  */
+  if (is_minimum_version(dbc->mysql.server_version, "4.1.1")
+      && odbc_stmt(dbc, "SET character_set_results = NULL") != SQL_SUCCESS)
+  {
+    return SQL_ERROR;
+  }
+
   return SQL_SUCCESS;
 }
 
@@ -199,20 +139,14 @@ SQLRETURN myodbc_set_initial_character_set(DBC *dbc, const char *charset)
   @return Standard SQLRETURN code. If it is @c SQL_SUCCESS or @c
   SQL_SUCCESS_WITH_INFO, a connection has been established.
 */
-SQLRETURN myodbc_do_connect(DBC *dbc, MYODBCUTIL_DATASOURCE *ds)
+SQLRETURN myodbc_do_connect(DBC *dbc, DataSource *ds)
 {
   SQLRETURN rc= SQL_SUCCESS;
   MYSQL *mysql= &dbc->mysql;
-  unsigned long options, flags;
-  unsigned int port;
+  unsigned long flags;
   /* Use 'int' and fill all bits to avoid alignment Bug#25920 */
   unsigned int opt_ssl_verify_server_cert = ~0;
-
-  /* Make sure default values are set on data source. */
-  MYODBCUtilDefaultDataSource(ds);
-
-  options= strtoul(ds->pszOPTION, NULL, 10);
-  port= (unsigned int)atoi(ds->pszPORT);
+  const my_bool on= 1;
 
 #ifdef WIN32
   /*
@@ -220,65 +154,128 @@ SQLRETURN myodbc_do_connect(DBC *dbc, MYODBCUTIL_DATASOURCE *ds)
    FLAG_COLUMN_SIZE_S32 option if we are.
   */
   if (GetModuleHandle("msado15.dll") != NULL)
-    options|= FLAG_COLUMN_SIZE_S32;
+    ds->limit_column_size= 1;
+
+  /* Detect another problem specific to MS Access */
+  if (GetModuleHandle("msaccess.exe") != NULL)
+    ds->default_bigint_bind_str= 1;
 #endif
 
   mysql_init(mysql);
 
-  flags= get_client_flags(options);
-
-  if (ds->bINTERACTIVE)
-  {
-    flags|= CLIENT_INTERACTIVE;
-  }
+  flags= get_client_flags(ds);
 
   /* Set other connection options */
 
-  if (options & (FLAG_BIG_PACKETS | FLAG_SAFE))
+  if (ds->allow_big_results || ds->safe)
     /* max_allowed_packet is a magical mysql macro. */
     max_allowed_packet= ~0L;
 
-  if (options & FLAG_NAMED_PIPE)
+  if (ds->force_use_of_named_pipes)
     mysql_options(mysql, MYSQL_OPT_NAMED_PIPE, NullS);
 
-  if (options & FLAG_USE_MYCNF)
+  if (ds->read_options_from_mycnf)
     mysql_options(mysql, MYSQL_READ_DEFAULT_GROUP, "odbc");
 
-  if (ds->pszSTMT && ds->pszSTMT[0])
-    mysql_options(mysql, MYSQL_INIT_COMMAND, ds->pszSTMT);
+  if (ds->initstmt && ds->initstmt[0])
+  {
+    /* Check for SET NAMES */
+    if (is_set_names_statement((SQLCHAR *)ds_get_utf8attr(ds->initstmt,
+                                                          &ds->initstmt8)))
+    {
+      return set_dbc_error(dbc, "HY000",
+                           "SET NAMES not allowed by driver", 0);
+    }
+    mysql_options(mysql, MYSQL_INIT_COMMAND, ds->initstmt8);
+  }
 
   if (dbc->login_timeout)
     mysql_options(mysql, MYSQL_OPT_CONNECT_TIMEOUT,
                   (char *)&dbc->login_timeout);
 
-  if (ds->pszREADTIMEOUT)
-  {
-    unsigned int to= strtoul(ds->pszREADTIMEOUT, NULL, 10);
-    mysql_options(mysql, MYSQL_OPT_READ_TIMEOUT, (const char *) &to);
-  }
+  if (ds->readtimeout)
+    mysql_options(mysql, MYSQL_OPT_READ_TIMEOUT,
+                  (const char *) &ds->readtimeout);
 
-  if (ds->pszWRITETIMEOUT)
-  {
-    unsigned int to= strtoul(ds->pszWRITETIMEOUT, NULL, 10);
-    mysql_options(mysql, MYSQL_OPT_WRITE_TIMEOUT, (const char *) &to);
-  }
+  if (ds->writetimeout)
+    mysql_options(mysql, MYSQL_OPT_WRITE_TIMEOUT,
+                  (const char *) &ds->writetimeout);
 
   /* set SSL parameters */
-  mysql_ssl_set(mysql, ds->pszSSLKEY, ds->pszSSLCERT, ds->pszSSLCA,
-                ds->pszSSLCAPATH, ds->pszSSLCIPHER);
-  if (ds->pszSSLVERIFY)
+  mysql_ssl_set(mysql,
+                ds_get_utf8attr(ds->sslkey,    &ds->sslkey8),
+                ds_get_utf8attr(ds->sslcert,   &ds->sslcert8),
+                ds_get_utf8attr(ds->sslca,     &ds->sslca8),
+                ds_get_utf8attr(ds->sslcapath, &ds->sslcapath8),
+                ds_get_utf8attr(ds->sslcipher, &ds->sslcipher8));
+
+  if (ds->sslverify)
     mysql_options(mysql, MYSQL_OPT_SSL_VERIFY_SERVER_CERT,
                   (const char *)&opt_ssl_verify_server_cert);
 
-  if (!mysql_real_connect(mysql, ds->pszSERVER, ds->pszUSER, ds->pszPASSWORD,
-                          ds->pszDATABASE, port, ds->pszSOCKET, flags))
   {
+    /*
+      Get the ANSI charset info before we change connection to UTF-8.
+    */
+    MY_CHARSET_INFO my_charset;
+    mysql_get_character_set_info(&dbc->mysql, &my_charset);
+    dbc->ansi_charset_info= get_charset(my_charset.number, MYF(0));
+
+  }
+
+  /*
+    We always use utf8 for the connection, and change it afterwards if needed.
+  */
+  mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "utf8");
+  dbc->cxn_charset_info= utf8_charset_info;
+
+#if MYSQL_VERSION_ID >= 50610
+  if (ds->can_handle_exp_pwd)
+  {
+    mysql_options(mysql, MYSQL_OPT_CAN_HANDLE_EXPIRED_PASSWORDS, (char *)&on);
+  }
+#endif
+
+  if (!mysql_real_connect(mysql,
+                          ds_get_utf8attr(ds->server,   &ds->server8),
+                          ds_get_utf8attr(ds->uid,      &ds->uid8),
+                          ds_get_utf8attr(ds->pwd,      &ds->pwd8),
+                          ds_get_utf8attr(ds->database, &ds->database8),
+                          ds->port,
+                          ds_get_utf8attr(ds->socket,   &ds->socket8),
+                          flags))
+  {
+
+#if MYSQL_VERSION_ID < 50610
+    /* In that special case when the driver was linked against old version of libmysql*/
+    if (mysql_errno(mysql) == ER_MUST_CHANGE_PASSWORD_LOGIN
+      && ds->can_handle_exp_pwd)
+    {
+      /* The password has expired, application said it knows how to deal with
+         that, but the driver was linked  that
+         does not support this option. Thus we change native error. */
+      /* TODO: enum/defines for driver specific errors */
+      return set_conn_error(dbc, MYERR_08004,
+        "Your password has expired, but underlying library doesn't support "
+        "this functionlaity", 0);
+    }
+#endif
     set_dbc_error(dbc, "HY000", mysql_error(mysql), mysql_errno(mysql));
     translate_error(dbc->error.sqlstate, MYERR_S1000, mysql_errno(mysql));
+
     return SQL_ERROR;
   }
 
-  if (!SQL_SUCCEEDED(myodbc_set_initial_character_set(dbc, ds->pszCHARSET)))
+  if (!is_minimum_version(dbc->mysql.server_version, "4.1.1"))
+  {
+    mysql_close(mysql);
+    set_dbc_error(dbc, "08001", "Driver does not support server versions under 4.1.1", 0);
+    return SQL_ERROR;
+  }
+
+  rc= myodbc_set_initial_character_set(dbc, ds_get_utf8attr(ds->charset,
+                                                            &ds->charset8));
+  if (!SQL_SUCCEEDED(rc))
   {
     /** @todo set failure reason */
     goto error;
@@ -290,28 +287,25 @@ SQLRETURN myodbc_do_connect(DBC *dbc, MYODBCUTIL_DATASOURCE *ds)
     unfortunately enabled by default. We have to turn it off, or it causes
     other problems.
   */
-  if (!(options & FLAG_AUTO_IS_NULL) &&
+  if (!ds->auto_increment_null_search &&
       odbc_stmt(dbc, "SET SQL_AUTO_IS_NULL = 0") != SQL_SUCCESS)
   {
     /** @todo set error reason */
     goto error;
   }
 
-  if (ds->pszDSN)
-    dbc->dsn= my_strdup(ds->pszDSN, MYF(MY_WME));
-  if (ds->pszSERVER)
-    dbc->server= my_strdup(ds->pszSERVER, MYF(MY_WME));
-  if (ds->pszUSER)
-    dbc->user= my_strdup(ds->pszUSER, MYF(MY_WME));
-  if (ds->pszPASSWORD)
-    dbc->password= my_strdup(ds->pszPASSWORD, MYF(MY_WME));
-  if (ds->pszDATABASE)
-    dbc->database= my_strdup(ds->pszDATABASE, MYF(MY_WME));
-
-  dbc->port= port;
-  dbc->flag= options;
-
-  if (options & FLAG_LOG_QUERY && !dbc->query_log)
+  dbc->ds= ds;
+  /* init all needed UTF-8 strings */
+  ds_get_utf8attr(ds->name, &ds->name8);
+  ds_get_utf8attr(ds->server, &ds->server8);
+  ds_get_utf8attr(ds->uid, &ds->uid8);
+  ds_get_utf8attr(ds->pwd, &ds->pwd8);
+  ds_get_utf8attr(ds->socket, &ds->socket8);
+  if (ds->database)
+    dbc->database= my_strdup(ds_get_utf8attr(ds->database, &ds->database8),
+                             MYF(MY_WME));
+  
+  if (ds->save_queries && !dbc->query_log)
     dbc->query_log= init_query_log();
 
   /* Set the statement error prefix based on the server version. */
@@ -319,16 +313,15 @@ SQLRETURN myodbc_do_connect(DBC *dbc, MYODBCUTIL_DATASOURCE *ds)
           mysql->server_version, "]", NullS);
 
   /* This needs to be set after connection, or it doesn't stick.  */
-  if (options & FLAG_AUTO_RECONNECT)
+  if (ds->auto_reconnect)
   {
-    my_bool reconnect= 1;
-    mysql_options(mysql, MYSQL_OPT_RECONNECT, (char *)&reconnect);
+    mysql_options(mysql, MYSQL_OPT_RECONNECT, (char *)&on);
   }
 
   /* Make sure autocommit is set as configured. */
   if (dbc->commit_flag == CHECK_AUTOCOMMIT_OFF)
   {
-    if (!trans_supported(dbc) || (options & FLAG_NO_TRANSACTIONS))
+    if (!trans_supported(dbc) || ds->disable_transactions)
     {
       rc= SQL_SUCCESS_WITH_INFO;
       dbc->commit_flag= CHECK_AUTOCOMMIT_ON;
@@ -398,27 +391,26 @@ error:
   Establish a connection to a data source.
 
   @param[in]  hdbc    Connection handle
-  @param[in]  szDSN   Data source name
-  @param[in]  cbDSN   Length of data source name or @c SQL_NTS
-  @param[in]  szUID   User identifier
-  @param[in]  cbUID   Length of user identifier or @c SQL_NTS
-  @param[in]  szAuth  Authentication string (password)
-  @param[in]  cbAuth  Length of authentication string or @c SQL_NTS
+  @param[in]  szDSN   Data source name (in connection charset)
+  @param[in]  cbDSN   Length of data source name in bytes or @c SQL_NTS
+  @param[in]  szUID   User identifier (in connection charset)
+  @param[in]  cbUID   Length of user identifier in bytes or @c SQL_NTS
+  @param[in]  szAuth  Authentication string (password) (in connection charset)
+  @param[in]  cbAuth  Length of authentication string in bytes or @c SQL_NTS
 
   @return  Standard ODBC success codes
 
   @since ODBC 1.0
   @since ISO SQL 92
 */
-SQLRETURN SQL_API SQLConnect(SQLHDBC  hdbc,
-                             SQLCHAR *szDSN,  SQLSMALLINT cbDSN,
-                             SQLCHAR *szUID, SQLSMALLINT cbUID,
-                             SQLCHAR *szAuth, SQLSMALLINT cbAuth)
+SQLRETURN SQL_API MySQLConnect(SQLHDBC   hdbc,
+                               SQLWCHAR *szDSN, SQLSMALLINT cbDSN,
+                               SQLWCHAR *szUID, SQLSMALLINT cbUID,
+                               SQLWCHAR *szAuth, SQLSMALLINT cbAuth)
 {
   SQLRETURN rc;
   DBC *dbc= (DBC *)hdbc;
-  char dsn[SQL_MAX_DSN_LENGTH], *dsn_ptr;
-  MYODBCUTIL_DATASOURCE *ds;
+  DataSource *ds;
 
 #ifdef NO_DRIVERMANAGER
   return set_dbc_error(dbc, "HY000",
@@ -432,46 +424,38 @@ SQLRETURN SQL_API SQLConnect(SQLHDBC  hdbc,
   /* Reset error state */
   CLEAR_DBC_ERROR(dbc);
 
-  dsn_ptr= fix_str(dsn, (char *)szDSN, cbDSN);
-
-  if (dsn_ptr && !dsn_ptr[0])
+  if (szDSN && !szDSN[0])
+  {
     return set_conn_error(hdbc, MYERR_S1000,
                           "Invalid connection parameters", 0);
-
-  ds= MYODBCUtilAllocDataSource(MYODBCUTIL_DATASOURCE_MODE_DRIVER_CONNECT);
-
-  /* Set username and password if they were provided. */
-  if (szUID && szUID[0])
-  {
-    if (cbUID == SQL_NTS)
-      cbUID= strlen((char *)szUID);
-    ds->pszUSER= _global_strndup((char *)szUID, cbUID);
-  }
-  if (szAuth && szAuth[0])
-  {
-    if (cbAuth == SQL_NTS)
-      cbAuth= strlen((char *)szAuth);
-    ds->pszPASSWORD= _global_strndup((char *)szAuth, cbAuth);
   }
 
-  /*
-    We don't care if this fails, because we might be able to get away
-    with the defaults. We probably should warn about this, though.
-  */
-  (void)MYODBCUtilReadDataSource(ds, dsn_ptr);
+  ds= ds_new();
+
+  ds_set_strnattr(&ds->name, szDSN, cbDSN);
+  ds_set_strnattr(&ds->uid, szUID, cbUID);
+  ds_set_strnattr(&ds->pwd, szAuth, cbAuth);
+
+  ds_lookup(ds);
 
   rc= myodbc_do_connect(dbc, ds);
 
-  MYODBCUtilFreeDataSource(ds);
+  if (!dbc->ds)
+    ds_delete(ds);
   return rc;
 #endif
 }
 
 
 /**
-  An alternative to SQLDriverConnect that allows specifying more of the
-  connection parameters, and whether or not to prompt the user for more
-  information using the setup library.
+  An alternative to SQLConnect that allows specifying more of the connection
+  parameters, and whether or not to prompt the user for more information
+  using the setup library.
+
+  NOTE: The prompting done in this function using MYODBCUTIL_DATASOURCE has
+        been observed to cause an access violation outside of our code
+        (specifically in Access after prompting, before table link list).
+        This has only been seen when setting a breakpoint on this function.
 
   @param[in]  hdbc  Handle of database connection
   @param[in]  hwnd  Window handle. May be @c NULL if no prompting will be done.
@@ -491,31 +475,29 @@ SQLRETURN SQL_API SQLConnect(SQLHDBC  hdbc,
   @since ODBC 1.0
   @since ISO SQL 92
 */
-SQLRETURN SQL_API SQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd,
-                                   SQLCHAR *szConnStrIn,
-                                   SQLSMALLINT cbConnStrIn
-                                     __attribute__((unused)),
-                                   SQLCHAR * szConnStrOut,
-                                   SQLSMALLINT cbConnStrOutMax,
-                                   SQLSMALLINT *pcbConnStrOut,
-                                   SQLUSMALLINT fDriverCompletion)
+SQLRETURN SQL_API MySQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd,
+                                     SQLWCHAR *szConnStrIn,
+                                     SQLSMALLINT cbConnStrIn,
+                                     SQLWCHAR *szConnStrOut,
+                                     SQLSMALLINT cbConnStrOutMax,
+                                     SQLSMALLINT *pcbConnStrOut,
+                                     SQLUSMALLINT fDriverCompletion)
 {
   SQLRETURN rc= SQL_SUCCESS;
   DBC *dbc= (DBC *)hdbc;
-  MYODBCUTIL_DATASOURCE *ds=
-    MYODBCUtilAllocDataSource(MYODBCUTIL_DATASOURCE_MODE_DRIVER_CONNECT);
+  DataSource *ds= ds_new();
   /* We may have to read driver info to find the setup library. */
-  MYODBCUTIL_DRIVER *pDriver= MYODBCUtilAllocDriver();
-  BOOL               bPrompt= FALSE;
-  HMODULE            hModule= NULL;
-  unsigned long      options;
-  SQLWCHAR          *outw=    NULL;
-  SQLWCHAR          *inw=     NULL;
-  SQLSMALLINT        outw_max, dummy_out;
-  uint               errors;
+  Driver *pDriver= driver_new();
+  SQLWCHAR *prompt_instr= NULL;
+  size_t prompt_inlen;
+  BOOL bPrompt= FALSE;
+  HMODULE hModule= NULL;
+
+  if (cbConnStrIn != SQL_NTS)
+    szConnStrIn= sqlwchardup(szConnStrIn, cbConnStrIn);
 
   /* Parse the incoming string */
-  if (!MYODBCUtilReadConnectStr(ds, (LPCSTR)szConnStrIn))
+  if (ds_from_kvpair(ds, szConnStrIn, (SQLWCHAR)';'))
   {
     rc= set_dbc_error(dbc, "HY000",
                       "Failed to parse the incoming connect string.", 0);
@@ -530,17 +512,13 @@ SQLRETURN SQL_API SQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd,
 
    This also allows us to get pszDRIVER (if not already given).
   */
-  if (ds->pszDSN)
-    (void)MYODBCUtilReadDataSource(ds, ds->pszDSN);
+  if (ds->name)
+    ds_lookup(ds);
 #endif
 
-  /* If FLAG_NO_PROMPT is no set, force prompting off. */
-  if (ds->pszOPTION)
-  {
-    options= strtoul(ds->pszOPTION, NULL, 10);
-    if (options & FLAG_NO_PROMPT)
-      fDriverCompletion= SQL_DRIVER_NOPROMPT;
-  }
+  /* If FLAG_NO_PROMPT is not set, force prompting off. */
+  if (ds->dont_prompt_upon_connect)
+    fDriverCompletion= SQL_DRIVER_NOPROMPT;
 
   /*
     We only prompt if we need to.
@@ -555,26 +533,17 @@ SQLRETURN SQL_API SQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd,
   switch (fDriverCompletion)
   {
   case SQL_DRIVER_PROMPT:
-    ds->nPrompt= MYODBCUTIL_DATASOURCE_PROMPT_PROMPT;
     bPrompt= TRUE;
     break;
 
   case SQL_DRIVER_COMPLETE:
-    ds->nPrompt= MYODBCUTIL_DATASOURCE_PROMPT_COMPLETE;
-    if (myodbc_do_connect(dbc, ds) == SQL_SUCCESS)
-      goto connected;
-    bPrompt= TRUE;
-    break;
-
   case SQL_DRIVER_COMPLETE_REQUIRED:
-    ds->nPrompt= MYODBCUTIL_DATASOURCE_PROMPT_REQUIRED;
     if (myodbc_do_connect(dbc, ds) == SQL_SUCCESS)
       goto connected;
     bPrompt= TRUE;
     break;
 
   case SQL_DRIVER_NOPROMPT:
-    ds->nPrompt= MYODBCUTIL_DATASOURCE_PROMPT_NOPROMPT;
     bPrompt= FALSE;
     break;
 
@@ -599,10 +568,7 @@ SQLRETURN SQL_API SQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd,
 
   if (bPrompt)
   {
-    PromptFunc    pFunc;
-    size_t        inw_len;
-    SQLCHAR       *prompt_str;
-    CHARSET_INFO  *clientcs= NULL;
+    PromptFunc pFunc;
 
     /*
      We can not present a prompt unless we can lookup the name of the setup
@@ -613,12 +579,13 @@ SQLRETURN SQL_API SQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd,
      connect with a DSN which does not exist. A possible solution would be to
      hard-code some fall-back value for ds->pszDRIVER.
     */
-    if (!ds->pszDRIVER && !ds->pszDriverFileName)
+    if (!ds->driver)
     {
       char szError[1024];
       sprintf(szError,
               "Could not determine the driver name; "
-              "could not lookup setup library. DSN=(%s)\n", ds->pszDSN);
+              "could not lookup setup library. DSN=(%s)\n",
+              ds_get_utf8attr(ds->name, &ds->name8));
       rc= set_dbc_error(hdbc, "HY000", szError, 0);
       goto error;
     }
@@ -631,28 +598,24 @@ SQLRETURN SQL_API SQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd,
       goto error;
     }
 
-    /*
-     At this point we should have a driver name (friendly name) either
-     loaded from DSN or provided in connection string. So lets determine
-     the setup library file name (better to not assume name). We read from
-     ODBC system info. This allows someone configure for a custom setup
-     interface.
-    */
-    if (!MYODBCUtilReadDriver(pDriver, ds->pszDRIVER, ds->pszDriverFileName))
+    /* if given a named DSN, we will have the path in the DRIVER field */
+    if (ds->name)
+        sqlwcharncpy(pDriver->lib, ds->driver, ODBCDRIVER_STRLEN);
+    /* otherwise, it's the driver name */
+    else
+        sqlwcharncpy(pDriver->name, ds->driver, ODBCDRIVER_STRLEN);
+
+    if (driver_lookup(pDriver))
     {
       char sz[1024];
-      if (ds->pszDRIVER && ds->pszDRIVER[0])
-        sprintf(sz, "Could not find driver '%s' in system information.",
-                ds->pszDRIVER);
-      else
-        sprintf(sz, "Could not find driver '%s' in system information.",
-                ds->pszDriverFileName);
+      sprintf(sz, "Could not find driver '%s' in system information.",
+              ds_get_utf8attr(ds->driver, &ds->driver8));
 
       rc= set_dbc_error(hdbc, "IM003", sz, 0);
       goto error;
     }
 
-    if (!pDriver->pszSETUP )
+    if (!*pDriver->setup_lib)
 #endif
     {
       rc= set_dbc_error(hdbc, "HY000",
@@ -667,24 +630,21 @@ SQLRETURN SQL_API SQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd,
     */
 #ifndef WIN32
 /*
-    dlinit();
+    lt_dlinit();
 */
 #endif
 
-    if (!(hModule= LoadLibrary(pDriver->pszSETUP)))
+    if (!(hModule= LoadLibrary(ds_get_utf8attr(pDriver->setup_lib,
+                                               &pDriver->setup_lib8))))
     {
       char sz[1024];
-      sprintf(sz, "Could not load the setup library '%s'.", pDriver->pszSETUP);
+      sprintf(sz, "Could not load the setup library '%s'.",
+              ds_get_utf8attr(pDriver->setup_lib, &pDriver->setup_lib8));
       rc= set_dbc_error(hdbc, "HY000", sz, 0);
       goto error;
     }
 
-    /*
-       The setup library should expose a MYODBCSetupDriverConnect() C entry point
-       for us to call.
-    */
-    pFunc= (PromptFunc)
-      GetProcAddress(hModule, "Driver_Prompt"/*"MYODBCSetupDriverConnect"*/);
+    pFunc= (PromptFunc)GetProcAddress(hModule, "Driver_Prompt");
 
     if (pFunc == NULL)
     {
@@ -703,153 +663,76 @@ SQLRETURN SQL_API SQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd,
       goto error;
     }
 
-    /**/
-    if (!utf8_charset_info)
+    /* create a string for prompting, and add driver manually */
+    prompt_inlen= ds_to_kvpair_len(ds) + sqlwcharlen(W_DRIVER_PARAM) +
+                  sqlwcharlen(ds->driver) + 1;
+    prompt_instr= (SQLWCHAR *) my_malloc(prompt_inlen * sizeof(SQLWCHAR),
+                                         MYF(0));
+    if (ds_to_kvpair(ds, prompt_instr, prompt_inlen, ';') == -1)
     {
-      utf8_charset_info= get_charset_by_csname("utf8", MYF(MY_CS_PRIMARY),
-                                             MYF(0));
+      rc= set_dbc_error(dbc, "HY000",
+                        "Failed to prepare prompt input.", 0);
+      goto error;
     }
-   
-    {
-      /* MYODBCUtilWriteDataSourceStrLen does not consider terminating null */
-      SQLINTEGER dummy_len= 1+ MYODBCUtilWriteDataSourceStrLen(ds, MYODBCUTIL_DELIM_SEMI);
-      SQLWCHAR *dummy;
-      const char *client_cs_name= NULL;
+    prompt_inlen-= sqlwcharlen(prompt_instr);
+    sqlwcharncat2(prompt_instr, W_DRIVER_PARAM, &prompt_inlen);
+    sqlwcharncat2(prompt_instr, ds->driver, &prompt_inlen);
 
-      prompt_str= (char *)my_malloc(dummy_len, MYF(0));
-
-      inw_len= dummy_len + sqlwcharlen(W_DRIVER_PARAM)
-              + strlen(pDriver->pszDRIVER) + 1;
-
-      inw= (SQLWCHAR *)my_malloc(sizeof(SQLWCHAR) * inw_len, MYF(0));
-      inw[0]= 0;
-
-      MYODBCUtilWriteDataSourceStr(ds, MYODBCUTIL_DELIM_SEMI, prompt_str,
-                                  (SQLSMALLINT)dummy_len);
-      prompt_str[dummy_len-1]= '\0';
-
-#ifdef _WIN32
-      {
-        char cpbuf[64];
-        my_snprintf(cpbuf, sizeof(cpbuf), "cp%u", GetACP());
-        client_cs_name= my_os_charset_to_mysql_charset(cpbuf);
-      }
-#endif
-
-      clientcs= client_cs_name
-             ? get_charset_by_csname(client_cs_name, MYF(MY_CS_PRIMARY), MYF(0))
-             : default_charset_info;
-
-
-      dummy= sqlchar_as_sqlwchar(clientcs, prompt_str, &dummy_len, &errors);
-      x_free(prompt_str);
-
-      sqlwcharncat2(inw, dummy, &inw_len);
-      x_free(dummy);
-
-      sqlwcharncat2(inw, W_DRIVER_PARAM, &inw_len);
-      dummy_len= strlen(pDriver->pszDRIVER);
-      dummy= sqlchar_as_sqlwchar(clientcs, pDriver->pszDRIVER,
-                                &dummy_len, &errors);
-      sqlwcharncat2(inw, dummy, &inw_len);
-      x_free(dummy)
-    }
-
-    outw_max= (sizeof(SQLWCHAR) * cbConnStrOutMax)/clientcs->mbmaxlen;
-    outw=     (SQLWCHAR *)my_malloc(sizeof(SQLWCHAR) * cbConnStrOutMax, MYF(0));
-
-    if (!pcbConnStrOut)
-    {
-      pcbConnStrOut= &dummy_out;
-    }
-
-    /* Prompt. Function returns false if user cancels.  */
-    if (!pFunc(hwnd, inw, fDriverCompletion, outw,
-            cbConnStrOutMax, pcbConnStrOut))
+    if (!pFunc(hwnd, prompt_instr, fDriverCompletion,
+               szConnStrOut, cbConnStrOutMax, pcbConnStrOut))
     {
       set_dbc_error(hdbc, "HY000", "User cancelled.", 0);
-      rc= ds->bSaveFileDSN ? SQL_NO_DATA : SQL_ERROR;
-
+      rc= SQL_NO_DATA;
       goto error;
     }
 
-    MYODBCUtilFreeDataSource(ds);
-    ds= MYODBCUtilAllocDataSource(MYODBCUTIL_DATASOURCE_MODE_DRIVER_CONNECT);
-
-    /* 3.51 parser cannot swallow named options from the 5.1 conn string */
+    /* refresh our DataSource */
+    ds_delete(ds);
+    ds= ds_new();
+    if (ds_from_kvpair(ds, szConnStrOut, ';'))
     {
-      /* TODO that now can be simplified by using */
-      DataSource *ds5= ds_new();
-
-      if (ds_from_kvpair(ds5, outw, ';'))
-      {
-        rc= set_dbc_error(dbc, "HY000",
-                          "Failed to parse the prompt output connect string.", 0);
-        goto error;
-      }
-
-      ds5_to_ds3(ds5, ds, clientcs);
-
-      ds_delete(ds5);
-    }
-  }
-
-  if (myodbc_do_connect(dbc, ds) != SQL_SUCCESS)
-  {
-    if (!ds->bSaveFileDSN)
-    {
-      /** @todo error message? */
-      rc= SQL_ERROR;
+      rc= set_dbc_error(dbc, "HY000",
+                        "Failed to parse the prompt output string.", 0);
       goto error;
     }
-    else
-    {
-      set_dbc_error(hdbc, "08001", "Client unable to establish connection.", 0);
-      rc= SQL_SUCCESS_WITH_INFO;
-    }
   }
 
+  if ((rc= myodbc_do_connect(dbc, ds)) != SQL_SUCCESS)
+    goto error;
+
 connected:
 
-  if (szConnStrOut)
+  /* copy input to output if connected without prompting */
+  if (!bPrompt && szConnStrOut && cbConnStrOutMax)
   {
-#ifdef WIN32
-    /*
-      Work around a bug in ADO/VB -- the intput and output strings are
-      set tot he same address, but the output string is not guaranteed
-      to have the recommended amount of space (1024 bytes).
-    */
-    if (szConnStrOut != szConnStrIn)
-#endif
-    {
-      *szConnStrOut= '\0';
-
-      if (!MYODBCUtilWriteConnectStr(ds, (char *)szConnStrOut, cbConnStrOutMax))
-      {
-        set_dbc_error(dbc, "01004",
-                      "String data, right truncated. "
-                      "Buffer for connection string was not large enough.", 0);
-        rc= SQL_SUCCESS_WITH_INFO;
-      }
-    }
-
+    size_t inlen= (sqlwcharlen(szConnStrIn) + 1) * sizeof(SQLWCHAR);
+    size_t copylen= myodbc_min((size_t)cbConnStrOutMax, inlen);
+    memcpy(szConnStrOut, szConnStrIn, copylen);
+    /* term needed if possibly truncated */
+    szConnStrOut[(copylen / sizeof(SQLWCHAR)) - 1] = 0;
     if (pcbConnStrOut)
-    {
-      *pcbConnStrOut= strlen((char *)szConnStrOut);
-    }
+      *pcbConnStrOut= (copylen / sizeof(SQLWCHAR)) - 1;
+  }
+
+  /* return SQL_SUCCESS_WITH_INFO if truncated output string */
+  if (pcbConnStrOut &&
+      cbConnStrOutMax - sizeof(SQLWCHAR) == *pcbConnStrOut * sizeof(SQLWCHAR))
+  {
+    set_dbc_error(hdbc, "01004", "String data, right truncated.", 0);
+    rc= SQL_SUCCESS_WITH_INFO;
   }
 
 error:
   if (hModule)
-  {
     FreeLibrary(hModule);
-  }
+  if (cbConnStrIn != SQL_NTS)
+    x_free(szConnStrIn);
 
-  MYODBCUtilFreeDriver(pDriver);
-  MYODBCUtilFreeDataSource(ds);
-
-  x_free(inw);
-  x_free(outw);
+  driver_delete(pDriver);
+  /* delete data source unless connected */
+  if (!dbc->ds)
+    ds_delete(ds);
+  x_free(prompt_instr);
 
   return rc;
 }
@@ -897,17 +780,16 @@ SQLRETURN SQL_API SQLDisconnect(SQLHDBC hdbc)
      my_SQLFreeStmt((SQLHSTMT)list_element->data, SQL_DROP);
   }
   mysql_close(&dbc->mysql);
-  x_free(dbc->dsn);
-  x_free(dbc->database);
-  x_free(dbc->server);
-  x_free(dbc->user);
-  x_free(dbc->password);
-  dbc->dsn= dbc->database= dbc->server= dbc->user= dbc->password= 0;
 
-  if (dbc->flag & FLAG_LOG_QUERY)
+  if (dbc->ds->save_queries)
     end_query_log(dbc->query_log);
 
+  x_free(dbc->database);
+  assert(dbc->ds);
+  ds_delete(dbc->ds);
+  dbc->ds= NULL;
+  dbc->database= NULL;
+
   return SQL_SUCCESS;
 }
 
-
