@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2001, 2011, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2001, 2012, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -43,6 +43,17 @@
 
 #include "driver.h"
 
+#ifdef _UNIX_
+/* variables for thread counter */
+static pthread_key_t myodbc_thread_counter_key;
+static pthread_once_t myodbc_thread_key_inited= PTHREAD_ONCE_INIT;
+
+/* Function to call pthread_key_create from pthread_once*/
+void myodbc_thread_key_create()
+{
+  pthread_key_create (&myodbc_thread_counter_key, 0);
+}
+#endif
 /*
   @type    : myodbc3 internal
   @purpose : to allocate the environment handle and to maintain
@@ -52,7 +63,9 @@
 SQLRETURN SQL_API my_SQLAllocEnv(SQLHENV FAR *phenv)
 {
 #ifdef _UNIX_
-    myodbc_init(); /* This is done in LibMain on XP so it probably needs to be in this func only when in UNIX - PAH */
+  /* Init thread key just once for all threads */
+  pthread_once(&myodbc_thread_key_inited, myodbc_thread_key_create);
+  myodbc_init();
 #endif
 
 #ifndef _UNIX_
@@ -160,14 +173,32 @@ SQLRETURN SQL_API my_SQLAllocConnect(SQLHENV henv, SQLHDBC FAR *phdbc)
     DBC FAR *dbc;
     ENV FAR *penv= (ENV FAR*) henv;
 
-#if MYSQL_VERSION_ID >= 40016
+#ifdef _UNIX_
+    long *thread_count;
+    thread_count= (long*)pthread_getspecific(myodbc_thread_counter_key);
+    
+    /* Increment or allocate the thread counter */
+    if (thread_count)
+    {
+      ++(*thread_count);
+    }
+    else
+    {
+      thread_count= my_malloc(sizeof(long), MYF(0));
+      (*thread_count)= 1;
+      pthread_setspecific(myodbc_thread_counter_key, thread_count);
+
+      /* Call it just for safety */
+      mysql_thread_init();
+    }
+#endif
+
     if (mysql_get_client_version() < MIN_MYSQL_VERSION)
     {
         char buff[255];
         sprintf(buff, "Wrong libmysqlclient library version: %ld.  MyODBC needs at least version: %ld", mysql_get_client_version(), MIN_MYSQL_VERSION);
         return(set_env_error(henv, MYERR_S1000, buff, 0));
     }
-#endif
 
     if (!penv->odbc_ver)
     {
@@ -207,11 +238,8 @@ SQLRETURN SQL_API my_SQLAllocConnect(SQLHENV henv, SQLHDBC FAR *phdbc)
 
     dbc= (DBC FAR*) *phdbc;
     dbc->mysql.net.vio= 0;     /* Marker if open */
-    dbc->flag= 0;
     dbc->commit_flag= 0;
     dbc->stmt_options.max_rows= dbc->stmt_options.max_length= 0L;
-    dbc->stmt_options.bind_type= SQL_BIND_BY_COLUMN;
-    dbc->stmt_options.rows_in_set= 1;
     dbc->stmt_options.cursor_type= SQL_CURSOR_FORWARD_ONLY;  /* ODBC default */
     dbc->login_timeout= 0;
     dbc->last_query_time= (time_t) time((time_t*) 0);
@@ -219,6 +247,9 @@ SQLRETURN SQL_API my_SQLAllocConnect(SQLHENV henv, SQLHDBC FAR *phdbc)
     dbc->env= penv;
     penv->connections= list_add(penv->connections,&dbc->list);
     dbc->list.data= dbc;
+    dbc->unicode= 0;
+    dbc->ansi_charset_info= dbc->cxn_charset_info= NULL;
+    dbc->exp_desc= NULL;
     dbc->sql_select_limit= (SQLULEN) -1;
     pthread_mutex_init(&dbc->lock,NULL);
     pthread_mutex_lock(&dbc->lock);
@@ -249,20 +280,52 @@ SQLRETURN SQL_API SQLAllocConnect(SQLHENV henv, SQLHDBC FAR *phdbc)
 SQLRETURN SQL_API my_SQLFreeConnect(SQLHDBC hdbc)
 {
     DBC FAR *dbc= (DBC FAR*) hdbc;
+    LIST *ldesc;
+    LIST *next;
 
     dbc->env->connections= list_delete(dbc->env->connections,&dbc->list);
-    x_free(dbc->dsn);
     x_free(dbc->database);
-    x_free(dbc->server);
-    x_free(dbc->user);
-    x_free(dbc->password);
+    if (dbc->ds)
+      ds_delete(dbc->ds);
     pthread_mutex_destroy(&dbc->lock);
 
+    /* free any remaining explicitly allocated descriptors */
+    for (ldesc= dbc->exp_desc; ldesc; ldesc= next)
+    {
+      next= ldesc->next;
+      desc_free((DESC *) ldesc->data);
+      x_free(ldesc);
+    }
+
 #ifndef _UNIX_
     GlobalUnlock(GlobalHandle((HGLOBAL) hdbc));
     GlobalFree(GlobalHandle((HGLOBAL) hdbc));
 #else
     x_free(hdbc);
+
+    {
+      long *thread_count;
+      thread_count= (long*)pthread_getspecific(myodbc_thread_counter_key);
+      
+      if (thread_count)
+      {
+        if (*thread_count)
+        {
+          --(*thread_count);
+        }
+
+        if (*thread_count == 0)
+        {
+          /* The value to the key must be reset before freeing the buffer */
+          pthread_setspecific(myodbc_thread_counter_key, 0);
+          x_free(thread_count);
+
+          /* Last connection deallocated, supposedly the thread is finishing */
+          mysql_thread_end();
+        }
+      }
+    }
+
 #endif
     return SQL_SUCCESS;
 }
@@ -296,20 +359,19 @@ SQLRETURN SQL_API my_SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT FAR *phstmt)
     hstmt= GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, sizeof(STMT));
     if (!hstmt || (*phstmt= (SQLHSTMT)GlobalLock(hstmt)) == SQL_NULL_HSTMT)
     {
-        GlobalFree(hstmt);
         *phstmt= SQL_NULL_HSTMT;
-        return SQL_ERROR;
+        GlobalFree(hstmt);
     }
 #else
     *phstmt= (SQLHSTMT) my_malloc(sizeof (STMT), MYF(MY_ZEROFILL | MY_WME));
-    if (*phstmt == SQL_NULL_HSTMT)
-    {
-        *phstmt= SQL_NULL_HSTMT;
-        return SQL_ERROR;
-    }
 #endif /* IS UNIX */
+    if (*phstmt == SQL_NULL_HSTMT)
+        goto error;
+
     stmt= (STMT FAR*) *phstmt;
     stmt->dbc= dbc;
+    stmt->ssps= NULL;;
+
     pthread_mutex_lock(&stmt->dbc->lock);
     dbc->statements= list_add(dbc->statements,&stmt->list);
     pthread_mutex_unlock(&stmt->dbc->lock);
@@ -317,12 +379,31 @@ SQLRETURN SQL_API my_SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT FAR *phstmt)
     stmt->stmt_options= dbc->stmt_options;
     stmt->state= ST_UNKNOWN;
     stmt->dummy_state= ST_DUMMY_UNKNOWN;
-#if !defined(DBUG_OFF) && defined(my_init_dynamic_array)
-    my_init_dynamic_array(&stmt->params,sizeof(PARAM_BIND),32,64);
-#else
-    init_dynamic_array(&stmt->params,sizeof(PARAM_BIND),32,64);
-#endif
+    strmov(stmt->error.sqlstate, "00000");
+    my_init_dynamic_array(&stmt->param_pos, sizeof(char *), 0, 0);
+
+    if (!(stmt->ard= desc_alloc(stmt, SQL_DESC_ALLOC_AUTO,
+                                DESC_APP, DESC_ROW)))
+        goto error;
+    if (!(stmt->ird= desc_alloc(stmt, SQL_DESC_ALLOC_AUTO,
+                                DESC_IMP, DESC_ROW)))
+        goto error;
+    if (!(stmt->apd= desc_alloc(stmt, SQL_DESC_ALLOC_AUTO,
+                                DESC_APP, DESC_PARAM)))
+        goto error;
+    if (!(stmt->ipd= desc_alloc(stmt, SQL_DESC_ALLOC_AUTO,
+                                DESC_IMP, DESC_PARAM)))
+        goto error;
+    stmt->imp_ard= stmt->ard;
+    stmt->imp_apd= stmt->apd;
+
     return SQL_SUCCESS;
+error:
+    x_free(stmt->ard);
+    x_free(stmt->ird);
+    x_free(stmt->apd);
+    x_free(stmt->ipd);
+    return set_dbc_error(dbc, "HY001", "Memory allocation error", MYERR_S1001);
 }
 
 
@@ -351,16 +432,6 @@ SQLRETURN SQL_API SQLFreeStmt(SQLHSTMT hstmt,SQLUSMALLINT fOption)
 }
 
 
-void odbc_reset_stmt_options(STMT_OPTIONS *options)
-{
-    reset_ptr(options->paramProcessedPtr);
-    reset_ptr(options->paramStatusPtr);
-    reset_ptr(options->rowOperationPtr);
-    reset_ptr(options->rowsFetchedPtr);
-    reset_ptr(options->rowStatusPtr);
-    reset_ptr(options->rowStatusPtr_ex);
-}
-
 /*
   @type    : myodbc3 internal
   @purpose : stops processing associated with a specific statement,
@@ -390,34 +461,37 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
 
     if (fOption == SQL_UNBIND)
     {
-        x_free(stmt->bind);
-        stmt->bind= 0;
-        stmt->bound_columns= 0;
-        return SQL_SUCCESS;
-    }
-    for (i= 0 ; i < stmt->params.elements ; i++)
-    {
-        PARAM_BIND *param= dynamic_element(&stmt->params,i,PARAM_BIND*);
-        if (param->alloced)
-        {
-            param->alloced= 0;
-            x_free(param->value);
-        }
-        if (fOption == SQL_RESET_PARAMS)
-        {
-            param->used= 0;
-            param->real_param_done= FALSE;
-        }
+      stmt->ard->records.elements= 0;
+      stmt->ard->count= 0;
+      return SQL_SUCCESS;
     }
+
+    desc_free_paramdata(stmt->apd);
+    /* reset data-at-exec state */
+    stmt->dae_type= 0;
+
+    scroller_reset(stmt);
+
     if (fOption == SQL_RESET_PARAMS)
-        return SQL_SUCCESS;
+    {
+      /* with current use of ps we need to close, if we prepare on server, we will have
+         to change this to mysql_stmt_reset
+         http://dev.mysql.com/doc/refman/5.5/en/mysql-stmt-reset.html
+       */
+      ssps_close(stmt);
+      /* remove all params and reset count to 0 (per spec) */
+      /* http://msdn2.microsoft.com/en-us/library/ms709284.aspx */
+      stmt->apd->count= 0;
+      return SQL_SUCCESS;
+    }
 
     if (!stmt->fake_result)
     {
-      mysql_free_result(stmt->result);
+      free_current_result(stmt);
       /* check if there are more resultsets */
       if (clearAllResults)
       {
+        /* For ps _close() will free all pending results */
         while (mysql_more_results(&stmt->dbc->mysql))
         {
           if (!mysql_next_result(&stmt->dbc->mysql))
@@ -429,36 +503,53 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
       }
     }
     else
+    {
+      if(stmt->result->field_alloc.pre_alloc)
+        free_root(&stmt->result->field_alloc, MYF(0));
+
       x_free(stmt->result);
+    }
+
     x_free(stmt->fields);
     x_free(stmt->array);
     x_free(stmt->result_array);
-    x_free(stmt->odbc_types);
+    x_free(stmt->lengths);
     stmt->result= 0;
     stmt->fake_result= 0;
-    stmt->result_lengths= 0;
     stmt->fields= 0;
     stmt->array= 0;
     stmt->result_array= 0;
-    stmt->odbc_types= 0;
+    stmt->lengths= 0;
     stmt->current_values= 0;   /* For SQLGetData */
     stmt->fix_fields= 0;
     stmt->affected_rows= 0;
     stmt->current_row= stmt->cursor_row= stmt->rows_found_in_set= 0;
-    stmt->state= ST_UNKNOWN;
+    stmt->dae_type= 0;
+    stmt->ird->count= 0;
 
     if (fOption == MYSQL_RESET_BUFFERS)
         return SQL_SUCCESS;
 
+    stmt->state= ST_UNKNOWN;
+
     x_free(stmt->table_name);
     stmt->table_name= 0;
     stmt->dummy_state= ST_DUMMY_UNKNOWN;
     stmt->cursor.pk_validated= FALSE;
+    if (stmt->setpos_apd)
+    {
+      desc_free(stmt->setpos_apd);
+    }
+    stmt->setpos_apd= NULL;
 
     for (i= stmt->cursor.pk_count; i--;)
-        stmt->cursor.pkcol[i].bind_done= 0;
+    {
+      stmt->cursor.pkcol[i].bind_done= 0;
+    }
     stmt->cursor.pk_count= 0;
 
+    ssps_close(stmt);
+
     if (fOption == SQL_CLOSE)
         return SQL_SUCCESS;
 
@@ -468,14 +559,30 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
     stmt->query= stmt->orig_query= 0;
     stmt->param_count= 0;
 
-    if (fOption == MYSQL_RESET)
-        return SQL_SUCCESS;
+    reset_ptr(stmt->apd->rows_processed_ptr);
+    reset_ptr(stmt->ard->rows_processed_ptr);
+    reset_ptr(stmt->ipd->array_status_ptr);
+    reset_ptr(stmt->ird->array_status_ptr);
+    reset_ptr(stmt->apd->array_status_ptr);
+    reset_ptr(stmt->ard->array_status_ptr);
+    reset_ptr(stmt->stmt_options.rowStatusPtr_ex);
 
-    odbc_reset_stmt_options(&stmt->stmt_options);
+    if (fOption == MYSQL_RESET)
+    {
+      return SQL_SUCCESS;
+    }
+
+    /* explicitly allocated descriptors are affected up until this point */
+    desc_remove_stmt(stmt->apd, stmt);
+    desc_remove_stmt(stmt->ard, stmt);
+    desc_free(stmt->imp_apd);
+    desc_free(stmt->imp_ard);
+    desc_free(stmt->ipd);
+    desc_free(stmt->ird);
 
     x_free(stmt->cursor.name);
-    x_free(stmt->bind);
-    delete_dynamic(&stmt->params);
+
+    delete_dynamic(&stmt->param_pos);
     pthread_mutex_lock(&stmt->dbc->lock);
     stmt->dbc->statements= list_delete(stmt->dbc->statements,&stmt->list);
     pthread_mutex_unlock(&stmt->dbc->lock);
@@ -489,6 +596,76 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
 }
 
 
+/*
+  Explicitly allocate a descriptor.
+*/
+SQLRETURN my_SQLAllocDesc(SQLHDBC hdbc, SQLHANDLE *pdesc)
+{
+  DBC *dbc= (DBC *) hdbc;
+  DESC *desc= desc_alloc(NULL, SQL_DESC_ALLOC_USER, DESC_APP, DESC_UNKNOWN);
+  LIST *e;
+
+  if (!desc)
+    return set_dbc_error(dbc, "HY001", "Memory allocation error", MYERR_S1001);
+
+  desc->exp.dbc= dbc;
+
+  /* add to this connection's list of explicit descriptors */
+  e= (LIST *) my_malloc(sizeof(LIST), MYF(0));
+  e->data= desc;
+  dbc->exp_desc= list_add(dbc->exp_desc, e);
+
+  *pdesc= desc;
+  return SQL_SUCCESS;
+}
+
+
+/*
+  Free an explicitly allocated descriptor. This resets all statements
+  that it was associated with to their implicitly allocated descriptors.
+*/
+SQLRETURN my_SQLFreeDesc(SQLHANDLE hdesc)
+{
+  DESC *desc= (DESC *) hdesc;
+  DBC *dbc= desc->exp.dbc;
+  LIST *lstmt;
+  LIST *ldesc;
+  LIST *next;
+
+  if (!desc)
+    return SQL_ERROR;
+  if (desc->alloc_type != SQL_DESC_ALLOC_USER)
+    return set_desc_error(desc, "HY017", "Invalid use of an automatically "
+                          "allocated descriptor handle.", MYERR_S1017);
+
+  /* remove from DBC */
+  for (ldesc= dbc->exp_desc; ldesc; ldesc= ldesc->next)
+  {
+    if (ldesc->data == desc)
+    {
+      dbc->exp_desc= list_delete(dbc->exp_desc, ldesc);
+      x_free(ldesc);
+      break;
+    }
+  }
+
+  /* reset all stmts it was on - to their implicit desc */
+  for (lstmt= desc->exp.stmts; lstmt; lstmt= next)
+  {
+    STMT *stmt= lstmt->data;
+    next= lstmt->next;
+    if (IS_APD(desc))
+      stmt->apd= stmt->imp_apd;
+    else if (IS_ARD(desc))
+      stmt->ard= stmt->imp_ard;
+    x_free(lstmt);
+  }
+
+  desc_free(desc);
+  return SQL_SUCCESS;
+}
+
+
 /*
   @type    : ODBC 3.0 API
   @purpose : allocates an environment, connection, statement, or
@@ -515,6 +692,10 @@ SQLRETURN SQL_API SQLAllocHandle(SQLSMALLINT HandleType,
             error= my_SQLAllocStmt(InputHandle,OutputHandlePtr);
             break;
 
+        case SQL_HANDLE_DESC:
+            error= my_SQLAllocDesc(InputHandle, OutputHandlePtr);
+            break;
+
         default:
             return set_conn_error(InputHandle,MYERR_S1C00,NULL,0);
     }
@@ -547,6 +728,12 @@ SQLRETURN SQL_API SQLFreeHandle(SQLSMALLINT HandleType,
         case SQL_HANDLE_STMT:
             error= my_SQLFreeStmt((STMT *)Handle, SQL_DROP);
             break;
+
+        case SQL_HANDLE_DESC:
+            error= my_SQLFreeDesc((DESC *) Handle);
+            break;
+
+
         default:
             break;
     }
@@ -554,3 +741,4 @@ SQLRETURN SQL_API SQLFreeHandle(SQLSMALLINT HandleType,
     return error;
 }
 
+
