@@ -1,29 +1,25 @@
 /*
-  Copyright 2003-2007 MySQL AB
-
-  The MySQL Connector/ODBC is licensed under the terms of the
-  GPL, like most MySQL Connectors. There are special exceptions
-  to the terms and conditions of the GPL as it is applied to
-  this software, see the FLOSS License Exception available on
-  mysql.com.
+  Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
+  The MySQL Connector/ODBC is licensed under the terms of the GPLv2
+  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
+  MySQL Connectors. There are special exceptions to the terms and
+  conditions of the GPLv2 as it is applied to this software, see the
+  FLOSS License Exception
+  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+  
   This program is free software; you can redistribute it and/or modify
-  it under the terms of version 2 of the GNU General Public License as
-  published by the Free Software Foundation.
-
-  There are special exceptions to the terms and conditions of the GPL
-  as it is applied to this software. View the full text of the exception
-  in file LICENSE.exceptions in the top-level directory of this software
-  distribution.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  it under the terms of the GNU General Public License as published
+  by the Free Software Foundation; version 2 of the License.
+  
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+  for more details.
+  
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 #include "odbctap.h"
@@ -745,7 +741,7 @@ DECLARE_TEST(t_bug10562)
   memset(blob, 'X', bsize);
 
   ok_sql(hstmt, "drop table if exists t_bug10562");
-  ok_sql(hstmt, "create table t_bug10562 ( id int not null primary key, mb longblob )");
+  ok_sql(hstmt, "create table t_bug10562 ( id int not null primary key DEFAULT 0, mb longblob )");
   ok_sql(hstmt, "insert into t_bug10562 (mb) values ('zzzzzzzzzz')");
 
   ok_sql(hstmt, "select id, mb from t_bug10562");
@@ -769,6 +765,70 @@ DECLARE_TEST(t_bug10562)
 }
 
 
+/* 
+  Bug#11746572: TEXT FIELDS WITH BINARY COLLATIONS 
+  Test for text field with latin1_bin and latin1_swedish_ci collation
+  Output of text column should contain same input value and not hexadecimal
+  value of input.
+*/
+DECLARE_TEST(t_bug_11746572)
+{
+  SQLCHAR     szData[MAX_ROW_DATA_LEN+1];
+  SQLSMALLINT SqlType;
+  SQLCHAR     ColName[MAX_NAME_LEN];
+
+  ok_sql(hstmt, "DROP TABLE if exists bug_11746572");
+
+  /* 
+    create table 'bug_11746572' with blob column and text columns 
+    with collation latin1_bin and latin1_swedish_ci.  
+  */
+  ok_sql(hstmt,"CREATE TABLE bug_11746572( blob_field BLOB ,"
+    "  text_bin TEXT CHARACTER SET latin1 COLLATE latin1_bin,"
+    "  text_def TEXT CHARACTER SET latin1 COLLATE latin1_swedish_ci)");
+
+  ok_sql(hstmt, "insert into bug_11746572 "
+          " set blob_field= 'blob', text_bin= 'text', "
+          " text_def= 'text' ; ");
+
+  ok_sql(hstmt, "SELECT * from bug_11746572");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  /* 
+    Verify inserted data is changed to hexadecimal value for blob field 
+    and remains unchanged for text field for both binary and non-binary 
+    collation.
+  */
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, szData, MAX_ROW_DATA_LEN,NULL));
+  is_str(szData, "626C6F62", 8);
+
+  ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_CHAR, szData, MAX_ROW_DATA_LEN,NULL));
+  is_str(szData, "text", 4);
+
+  ok_stmt(hstmt, SQLGetData(hstmt, 3, SQL_C_CHAR, szData, MAX_ROW_DATA_LEN,NULL));
+  is_str(szData, "text", 4);
+
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 1, ColName, MAX_NAME_LEN, 
+                        NULL, &SqlType, NULL, NULL, NULL));
+  is_num(SqlType, SQL_LONGVARBINARY);
+
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 2, ColName, MAX_NAME_LEN, 
+                        NULL, &SqlType, NULL, NULL, NULL));
+  is_num(SqlType, SQL_WLONGVARCHAR);
+
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 3, ColName, MAX_NAME_LEN, 
+                        NULL, &SqlType, NULL, NULL, NULL));
+  is_num(SqlType, SQL_LONGVARCHAR);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_sql(hstmt, "DROP TABLE bug_11746572");
+
+  return OK;
+}
+
+
 BEGIN_TESTS
   ADD_TEST(t_blob)
   ADD_TEST(t_1piecewrite2)
@@ -781,6 +841,7 @@ BEGIN_TESTS
   ADD_TEST(getdata_lenonly)
   ADD_TEST(t_bug9781)
   ADD_TEST(t_bug10562)
+  ADD_TEST(t_bug_11746572)
 END_TESTS
 
 
