@@ -22,6 +22,7 @@
   51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+#include <time.h>
 #include "odbctap.h"
 
 
@@ -115,75 +116,92 @@ DECLARE_TEST(my_ts)
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS my_ts");
 
+  /* Test of 2-digits year(YYMMDD) format */
+  ok_sql(hstmt, "select \"910825\""); 
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_TIMESTAMP, &ts, sizeof(ts), &len));
+
+  is_num(1991, ts.year);
+  is_num(8, ts.month);
+  is_num(25, ts.day);
+
   return OK;
 }
 
 
 DECLARE_TEST(t_tstotime)
 {
-    SQLRETURN rc;
-    SQL_TIMESTAMP_STRUCT ts;
+  SQLRETURN rc;
+  SQL_TIMESTAMP_STRUCT ts, ts1, ts2;
 
-    ts.day    = 02;
-    ts.month  = 8;
-    ts.year   = 2001;
-    ts.hour   = 18;
-    ts.minute = 20;
-    ts.second = 45;
-    ts.fraction = 05;   
+  ts.day    = 02;
+  ts.month  = 8;
+  ts.year   = 2001;
+  ts.hour   = 18;
+  ts.minute = 20;
+  ts.second = 45;
+  ts.fraction = 05;   
+
+  memcpy(&ts1, (void*) &ts, sizeof(SQL_TIMESTAMP_STRUCT));
+  /* For SQL_TIME fraction is truncated and that would cause error */
+  ts1.fraction= 0;
+
+  memcpy(&ts2, (void*) &ts1, sizeof(SQL_TIMESTAMP_STRUCT));
+  /* Same for SQL_DATE - time is truncated -> error */
+  ts2.hour= ts2.minute= ts2.second= 0;
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_tstotime");
 
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
+  mycon(hdbc,rc);
 
-    rc = tmysql_exec(hstmt,"create table t_tstotime(col1 date ,col2 time, col3 timestamp)");
-    mystmt(hstmt,rc);
+  rc = tmysql_exec(hstmt,"create table t_tstotime(col1 date ,col2 time, col3 timestamp)");
+  mystmt(hstmt,rc);
 
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
+  mycon(hdbc,rc);
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+  rc = SQLFreeStmt(hstmt,SQL_CLOSE);
+  mystmt(hstmt,rc);
 
-    /* TIMESTAMP TO DATE, TIME and TS CONVERSION */
-    rc = SQLPrepare(hstmt,"insert into t_tstotime(col1,col2,col3) values(?,?,?)",SQL_NTS);
-    mystmt(hstmt,rc);   
+  /* TIMESTAMP TO DATE, TIME and TS CONVERSION */
+  rc = SQLPrepare(hstmt, (SQLCHAR *)"insert into t_tstotime(col1,col2,col3) values(?,?,?)",SQL_NTS);
+  mystmt(hstmt,rc);   
 
-    rc = SQLBindParameter(hstmt,1,SQL_PARAM_INPUT,SQL_C_TIMESTAMP,
-                          SQL_DATE,0,0,&ts,sizeof(ts),NULL);
-    mystmt(hstmt,rc);
+  rc = SQLBindParameter(hstmt,1,SQL_PARAM_INPUT,SQL_C_TIMESTAMP,
+                        SQL_DATE,0,0,&ts2,sizeof(ts2),NULL);
+  mystmt(hstmt,rc);
 
-    rc = SQLBindParameter(hstmt,2,SQL_PARAM_INPUT,SQL_C_TIMESTAMP,
-                          SQL_TIME,0,0,&ts,sizeof(ts),NULL);
-    mystmt(hstmt,rc);
+  rc = SQLBindParameter(hstmt,2,SQL_PARAM_INPUT,SQL_C_TIMESTAMP,
+                        SQL_TIME,0,0,&ts1,sizeof(ts1),NULL);
+  mystmt(hstmt,rc);
 
-    rc = SQLBindParameter(hstmt,3,SQL_PARAM_INPUT,SQL_C_TIMESTAMP,
-                          SQL_TIMESTAMP,0,0,&ts,sizeof(ts),NULL);
-    mystmt(hstmt,rc);
+  rc = SQLBindParameter(hstmt,3,SQL_PARAM_INPUT,SQL_C_TIMESTAMP,
+                        SQL_TIMESTAMP,0,0,&ts,sizeof(ts),NULL);
+  mystmt(hstmt,rc);
 
-    rc = SQLExecute(hstmt);
-    mystmt(hstmt,rc);
+  rc = SQLExecute(hstmt);
+  mystmt(hstmt,rc);
 
-    rc = SQLFreeStmt(hstmt,SQL_RESET_PARAMS);
-    mystmt(hstmt,rc);  
+  rc = SQLFreeStmt(hstmt,SQL_RESET_PARAMS);
+  mystmt(hstmt,rc);  
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);  
+  rc = SQLFreeStmt(hstmt,SQL_CLOSE);
+  mystmt(hstmt,rc);  
 
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
+  mycon(hdbc,rc);
 
-    rc = tmysql_exec(hstmt,"select * from t_tstotime");
-    mystmt(hstmt,rc);  
+  rc = tmysql_exec(hstmt,"select * from t_tstotime");
+  mystmt(hstmt,rc);  
 
-    my_assert( 1 == myresult(hstmt));
+  my_assert( 1 == myresult(hstmt));
 
-    rc = SQLFreeStmt(hstmt,SQL_UNBIND);
-    mystmt(hstmt,rc);
+  rc = SQLFreeStmt(hstmt,SQL_UNBIND);
+  mystmt(hstmt,rc);
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+  rc = SQLFreeStmt(hstmt,SQL_CLOSE);
+  mystmt(hstmt,rc);
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_tstotime");
 
@@ -204,15 +222,33 @@ DECLARE_TEST(t_tstotime1)
 
   /* TIMESTAMP TO DATE, TIME and TS CONVERSION */
   ok_stmt(hstmt, SQLPrepare(hstmt,
-                            "INSERT INTO t_tstotime1  VALUES (?,?,?)",
+                            (SQLCHAR *)"INSERT INTO t_tstotime1 VALUES (?,?,?)",
                             SQL_NTS));
 
-  ok_stmt(hstmt, SQLBindParameter(hstmt,1,SQL_PARAM_INPUT,SQL_C_CHAR,
-                                  SQL_DATE,0,0,&ts,sizeof(ts),NULL));
-  ok_stmt(hstmt, SQLBindParameter(hstmt,2,SQL_PARAM_INPUT,SQL_C_CHAR,
-                                  SQL_TIME,0,0,&ts,sizeof(ts),NULL));
-  ok_stmt(hstmt, SQLBindParameter(hstmt,3,SQL_PARAM_INPUT,SQL_C_CHAR,
-                                  SQL_TIMESTAMP,0,0,&ts,sizeof(ts),NULL));
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                  SQL_DATE, 0, 0, &ts, sizeof(ts), NULL));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                  SQL_TIME, 0, 0, &ts, sizeof(ts), NULL));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 3, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                  SQL_TIMESTAMP, 0, 0, &ts, sizeof(ts), NULL));
+
+  expect_stmt(hstmt, SQLExecute(hstmt), SQL_ERROR);
+
+  is_num(check_sqlstate(hstmt, "22008"), OK);
+
+  /* Taking only date part */
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                  SQL_DATE, 0, 0, &ts, 10, NULL));
+
+  expect_stmt(hstmt, SQLExecute(hstmt), SQL_ERROR);
+
+  is_num(check_sqlstate(hstmt, "22008"), OK);
+
+  /* are not taking fractional part */
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                  SQL_TIME, 0, 0, &ts, 19, NULL));
 
   ok_stmt(hstmt, SQLExecute(hstmt));
 
@@ -286,58 +322,57 @@ DECLARE_TEST(t_time)
   SQLCHAR         str[20];
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_time");
-    rc = tmysql_exec(hstmt,"create table t_time(tm time, ts timestamp)");
-    mystmt(hstmt,rc);
+  rc = tmysql_exec(hstmt,"create table t_time(tm time, ts timestamp)");
+  mystmt(hstmt,rc);
 
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
+  mycon(hdbc,rc);
 
-    rc = SQLPrepare(hstmt,"insert into t_time values (?,?)",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLPrepare(hstmt,
+                            (SQLCHAR *)"insert into t_time values (?,?)",
+                            SQL_NTS));
 
-    rc = SQLBindParameter( hstmt, 1, SQL_PARAM_INPUT, SQL_C_TIME,
-                           SQL_TIME, 0, 0, &tm, 0, NULL );
-    mystmt(hstmt,rc);
+  rc = SQLBindParameter( hstmt, 1, SQL_PARAM_INPUT, SQL_C_TIME,
+                         SQL_TIME, 0, 0, &tm, 0, NULL );
+  mystmt(hstmt,rc);
 
-    rc = SQLBindParameter( hstmt, 2, SQL_PARAM_INPUT, SQL_C_TIME,
-                           SQL_TIMESTAMP, 0, 0, &tm, 15, NULL );
-    mystmt(hstmt,rc);
+  rc = SQLBindParameter( hstmt, 2, SQL_PARAM_INPUT, SQL_C_TIME,
+                         SQL_TIMESTAMP, 0, 0, &tm, 15, NULL );
+  mystmt(hstmt,rc);
 
-    tm.hour = 20;
-    tm.minute = 59;
-    tm.second = 45;
+  tm.hour = 20;
+  tm.minute = 59;
+  tm.second = 45;
 
-    rc = SQLExecute(hstmt);
-    mystmt(hstmt,rc);
+  rc = SQLExecute(hstmt);
+  mystmt(hstmt,rc);
 
-    rc = SQLFreeStmt(hstmt,SQL_RESET_PARAMS);
-    mystmt(hstmt,rc);
+  rc = SQLFreeStmt(hstmt,SQL_RESET_PARAMS);
+  mystmt(hstmt,rc);
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+  rc = SQLFreeStmt(hstmt,SQL_CLOSE);
+  mystmt(hstmt,rc);
 
-    rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
-    mycon(hdbc,rc);
+  rc = SQLTransact(NULL,hdbc,SQL_COMMIT);
+  mycon(hdbc,rc);
 
-    rc = SQLExecDirect(hstmt,"select tm from t_time",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "select tm from t_time");
 
-    rc = SQLFetch(hstmt);
-    mystmt(hstmt,rc);
+  rc = SQLFetch(hstmt);
+  mystmt(hstmt,rc);
 
-    rc = SQLGetData(hstmt,1,SQL_C_CHAR,&str,100,NULL);
-    mystmt(hstmt,rc);
-    fprintf(stdout,"time:%s\n",str);
-    my_assert(strcmp(str,"20:59:45")==0);
+  rc = SQLGetData(hstmt,1,SQL_C_CHAR,&str,100,NULL);
+  mystmt(hstmt,rc);
+  is_str(str, "20:59:45", 9);
 
-    rc = SQLFetch(hstmt);
-    my_assert(rc == SQL_NO_DATA_FOUND);
+  rc = SQLFetch(hstmt);
+  my_assert(rc == SQL_NO_DATA_FOUND);
 
-    rc = SQLFreeStmt(hstmt,SQL_UNBIND);
-    mystmt(hstmt,rc);
+  rc = SQLFreeStmt(hstmt,SQL_UNBIND);
+  mystmt(hstmt,rc);
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+  rc = SQLFreeStmt(hstmt,SQL_CLOSE);
+  mystmt(hstmt,rc);
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_time");
 
@@ -354,11 +389,11 @@ DECLARE_TEST(t_time1)
   SQLLEN          length;
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_time");
-    rc = SQLExecDirect(hstmt,"create table t_time(t time, t1 timestamp, t2 datetime, t3 date)",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_sql(hstmt, "create table t_time(t time, t1 timestamp, t2 datetime, t3 date)");
 
-    rc = SQLPrepare(hstmt,"insert into t_time(t) values(?)",SQL_NTS);
-    mystmt(hstmt,rc);
+  ok_stmt(hstmt, SQLPrepare(hstmt,
+                            (SQLCHAR *)"insert into t_time(t) values(?)",
+                            SQL_NTS));
 
     rc = SQLBindParameter(hstmt,1,SQL_PARAM_INPUT,SQL_C_TYPE_TIME,
                           SQL_TIME,0,0,&tt,0,NULL);
@@ -437,99 +472,87 @@ DECLARE_TEST(t_time1)
     SQLFreeStmt(hstmt, SQL_RESET_PARAMS);
     SQLFreeStmt(hstmt, SQL_CLOSE);
 
-    rc = SQLExecDirect(hstmt,"select t from t_time",SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt, "select t from t_time");
 
     rc = SQLFetch(hstmt);
     mystmt(hstmt,rc);
 
     rc = SQLGetData(hstmt, 1, SQL_C_CHAR, data, sizeof(data), &length);
     mystmt(hstmt,rc);
-    fprintf(stdout,"time: %s(%d)\n", data, length);
-    myassert(strcmp(data,"00:00:03")==0);
-    myassert(length == 8);
+    is_num(length, 8);
+    is_str(data, "00:00:03", 9);
 
     rc = SQLFetch(hstmt);
     mystmt(hstmt,rc);
 
     rc = SQLGetData(hstmt, 1, SQL_C_CHAR, data, sizeof(data), &length);
     mystmt(hstmt,rc);
-    fprintf(stdout,"time: %s(%d)\n", data, length);
-    myassert(strcmp(data,"01:00:00")==0);
-    myassert(length == 8);
+    is_num(length, 8);
+    is_str(data, "01:00:00", 9);
 
     rc = SQLFetch(hstmt);
     mystmt(hstmt,rc);
 
     rc = SQLGetData(hstmt, 1, SQL_C_CHAR, data, sizeof(data), &length);
     mystmt(hstmt,rc);
-    fprintf(stdout,"time: %s(%d)\n", data, length);
-    myassert(strcmp(data,"19:00:00")==0);
-    myassert(length == 8);
+    is_num(length, 8);
+    is_str(data, "19:00:00", 9);
 
     rc = SQLFetch(hstmt);
     mystmt(hstmt,rc);
 
     rc = SQLGetData(hstmt, 1, SQL_C_CHAR, data, sizeof(data), &length);
     mystmt(hstmt,rc);
-    fprintf(stdout,"time: %s(%d)\n", data, length);
-    myassert(strcmp(data,"01:01:00")==0);
-    myassert(length == 8);
+    is_num(length, 8);
+    is_str(data, "01:01:00", 9);
 
     rc = SQLFetch(hstmt);
     mystmt(hstmt,rc);
 
     rc = SQLGetData(hstmt, 1, SQL_C_CHAR, data, sizeof(data), &length);
     mystmt(hstmt,rc);
-    fprintf(stdout,"time: %s(%d)\n", data, length);
-    myassert(strcmp(data,"01:00:01")==0);
-    myassert(length == 8);
+    is_num(length, 8);
+    is_str(data, "01:00:01", 9);
 
     rc = SQLFetch(hstmt);
     mystmt(hstmt,rc);
 
     rc = SQLGetData(hstmt, 1, SQL_C_CHAR, data, sizeof(data), &length);
     mystmt(hstmt,rc);
-    fprintf(stdout,"time: %s(%d)\n", data, length);
-    myassert(strcmp(data,"00:01:00")==0);
-    myassert(length == 8);
+    is_num(length, 8);
+    is_str(data, "00:01:00", 9);
 
     rc = SQLFetch(hstmt);
     mystmt(hstmt,rc);
 
     rc = SQLGetData(hstmt, 1, SQL_C_CHAR, data, sizeof(data), &length);
     mystmt(hstmt,rc);
-    fprintf(stdout,"time: %s(%d)\n", data, length);
-    myassert(strcmp(data,"00:11:12")==0);
-    myassert(length == 8);
+    is_num(length, 8);
+    is_str(data, "00:11:12", 9);
 
     rc = SQLFetch(hstmt);
     mystmt(hstmt,rc);
 
     rc = SQLGetData(hstmt, 1, SQL_C_CHAR, data, sizeof(data), &length);
     mystmt(hstmt,rc);
-    fprintf(stdout,"time: %s(%d)\n", data, length);
-    myassert(strcmp(data,"01:01:01")==0);
-    myassert(length == 8);
+    is_num(length, 8);
+    is_str(data, "01:01:01", 9);
 
     rc = SQLFetch(hstmt);
     mystmt(hstmt,rc);
 
     rc = SQLGetData(hstmt, 1, SQL_C_CHAR, data, sizeof(data), &length);
     mystmt(hstmt,rc);
-    fprintf(stdout,"time: %s(%d)\n", data, length);
-
-    myassert(strcmp(data,"00:00:00")==0);
-    myassert(length == 8);
+    is_num(length, 8);
+    is_str(data, "00:00:00", 9);
 
     rc = SQLFetch(hstmt);
     mystmt(hstmt,rc);
 
     rc = SQLGetData(hstmt, 1, SQL_C_CHAR, data, sizeof(data), &length);
     mystmt(hstmt,rc);
-    fprintf(stdout,"time: %s(%d)\n", data, length);
-    myassert(strcmp(data,"10:11:12")==0);
-    myassert(length == 8);
+    is_num(length, 8);
+    is_str(data, "10:11:12", 9);
 
     rc = SQLFetch(hstmt);
     myassert(rc == SQL_NO_DATA);
@@ -537,24 +560,21 @@ DECLARE_TEST(t_time1)
     SQLFreeStmt(hstmt, SQL_UNBIND);
     SQLFreeStmt(hstmt, SQL_CLOSE);
 
-    rc = SQLExecDirect(hstmt,"delete from t_time",SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt,"delete from t_time");
 
-    rc = SQLExecDirect(hstmt,"insert into t_time(t1) values('2003-05-12 10:11:12')",SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt, "insert into t_time(t1) values('2003-05-12 10:11:12')");
 
-    rc = SQLExecDirect(hstmt,"select t1 from t_time", SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt, "select t1 from t_time");
 
     rc = SQLFetch(hstmt);
     mystmt(hstmt,rc);
 
     rc = SQLGetData(hstmt, 1, SQL_C_TIME, &tt, sizeof(tt), &length);
     mystmt(hstmt,rc);
-    fprintf(stdout,"time: %d:%d:%d(%d)\n", tt.hour, tt.minute, tt.second, length);
-
-    myassert(tt.hour == 10 && tt.minute == 11 && tt.second == 12);
-    myassert(length == sizeof(SQL_TIME_STRUCT));
+    is_num(tt.hour, 10);
+    is_num(tt.minute, 11);
+    is_num(tt.second, 12);
+    is_num(length, sizeof(SQL_TIME_STRUCT));
 
     rc = SQLFetch(hstmt);
     myassert(rc == SQL_NO_DATA);
@@ -562,47 +582,39 @@ DECLARE_TEST(t_time1)
     SQLFreeStmt(hstmt, SQL_UNBIND);
     SQLFreeStmt(hstmt, SQL_CLOSE);
 
-    rc = SQLExecDirect(hstmt,"delete from t_time",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect(hstmt,"insert into t_time(t2) values('03-12-28 05:59:59')",SQL_NTS);
-    mystmt(hstmt,rc);
-
-    rc = SQLExecDirect(hstmt,"select t2 from t_time", SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt,"delete from t_time");
+    ok_sql(hstmt,"insert into t_time(t2) values('03-12-28 05:59:59')");
+    ok_sql(hstmt,"select t2 from t_time");
 
     rc = SQLFetch(hstmt);
     mystmt(hstmt,rc);
 
     rc = SQLGetData(hstmt, 1, SQL_C_TIME, &tt, sizeof(tt), &length);
     mystmt(hstmt,rc);
-    fprintf(stdout,"time: %d:%d:%d(%d)\n", tt.hour, tt.minute, tt.second, length);
-    myassert(tt.hour == 05 && tt.minute == 59 && tt.second == 59);
-    myassert(length == sizeof(SQL_TIME_STRUCT));
+    is_num(tt.hour, 05);
+    is_num(tt.minute, 59);
+    is_num(tt.second, 59);
+    is_num(length, sizeof(SQL_TIME_STRUCT));
 
     rc = SQLFetch(hstmt);
-    myassert(rc == SQL_NO_DATA);
+    is_num(rc, SQL_NO_DATA);
 
     SQLFreeStmt(hstmt, SQL_UNBIND);
     SQLFreeStmt(hstmt, SQL_CLOSE);
 
-    rc = SQLExecDirect(hstmt,"delete from t_time",SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt,"delete from t_time");
 
-    rc = SQLExecDirect(hstmt,"insert into t_time(t3) values('2003-05-12 10:11:12')",SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt,"insert into t_time(t3) values('2003-05-12 10:11:12')");
 
-    rc = SQLExecDirect(hstmt,"select t3 from t_time", SQL_NTS);
-    mystmt(hstmt,rc);
+    ok_sql(hstmt,"select t3 from t_time");
 
     rc = SQLFetch(hstmt);
     mystmt(hstmt,rc);
 
     rc = SQLGetData(hstmt, 1, SQL_C_TIME, &tt, sizeof(tt), &length);
     mystmt(hstmt,rc);
-    fprintf(stdout,"time: %d:%d:%d(%d)\n", tt.hour, tt.minute, tt.second, length);
-    myassert(tt.hour == 00 || tt.minute == 00 || tt.second == 00);
-    myassert(length == sizeof(SQL_TIME_STRUCT));
+    is(tt.hour == 00 || tt.minute == 00 || tt.second == 00);
+    is_num(length, sizeof(SQL_TIME_STRUCT));
 
     rc = SQLFetch(hstmt);
     myassert(rc == SQL_NO_DATA);
@@ -732,6 +744,9 @@ DECLARE_TEST(t_bug9927)
 */
 DECLARE_TEST(t_bug30081)
 {
+  if (!mysql_min_version(hdbc, "5.1.23", 6))
+    skip("necessary feature added in MySQL 5.1.23");
+
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug30081");
   ok_sql(hstmt,
          "CREATE TABLE t_bug30081 (a TIMESTAMP DEFAULT 0,"
@@ -758,7 +773,7 @@ DECLARE_TEST(t_bug30081)
 DECLARE_TEST(t_datecolumns)
 {
   SQLCHAR col[10];
-  SQLINTEGER type;
+  SQLLEN type;
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_datecolumns");
   ok_sql(hstmt,
@@ -857,16 +872,16 @@ DECLARE_TEST(t_bug14414)
   */
   ok_sql(hstmt, "SELECT * FROM t_bug14414");
 
-  ok_stmt(hstmt, SQLDescribeCol(hstmt, 1, NULL, 0, NULL, NULL, NULL, NULL,
-                                &nullable));
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 1, col, sizeof(col), NULL, NULL, NULL,
+                                NULL, &nullable));
   is_num(nullable, SQL_NULLABLE);
 
-  ok_stmt(hstmt, SQLDescribeCol(hstmt, 2, NULL, 0, NULL, NULL, NULL, NULL,
-                                &nullable));
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 2, col, sizeof(col), NULL, NULL, NULL,
+                                NULL, &nullable));
   is_num(nullable, SQL_NULLABLE);
 
-  ok_stmt(hstmt, SQLDescribeCol(hstmt, 3, NULL, 0, NULL, NULL, NULL, NULL,
-                                &nullable));
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 3, col, sizeof(col), NULL, NULL, NULL,
+                                NULL, &nullable));
   is_num(nullable, SQL_NULLABLE);
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
@@ -927,6 +942,265 @@ DECLARE_TEST(t_bug31009)
 }
 
 
+/**
+ Bug #37342: ODBC TIMESTAMP string format not handled properly by ODBC driver
+*/
+DECLARE_TEST(t_bug37342)
+{
+  SQLCHAR *date= (SQLCHAR *)"{dt '2007-01-13'}";
+  SQLCHAR *time= (SQLCHAR *)"194759";
+  SQLCHAR out[30];
+  TIMESTAMP_STRUCT ts;
+  SQLLEN len= SQL_NTS;
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                  SQL_TIMESTAMP, 0, 0, date, 0, &len));
+
+  ok_sql(hstmt, "SELECT ? AS foo");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_str(my_fetch_str(hstmt, out, 1), "2007-01-13", 11);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                  SQL_TYPE_TIME, 0, 0, time, 0, &len));
+
+  ok_sql(hstmt, "SELECT ? AS foo");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_str(my_fetch_str(hstmt, out, 1), "19:47:59", 9);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_TIMESTAMP,
+                                  SQL_TYPE_TIME, 0, 0, &ts, sizeof(ts), NULL));
+
+  ts.hour= 19;
+  ts.minute= 47;
+  ts.second= 59;
+  ts.fraction= 4;
+
+  /* Fractional truncation */
+  expect_sql(hstmt, "SELECT ? AS foo", SQL_ERROR);
+  ts.fraction= 0;
+
+  ok_sql(hstmt, "SELECT ? AS foo");
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  is_str(my_fetch_str(hstmt, out, 1), "19:47:59", 9);
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA);
+
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  return OK;
+}
+
+
+/**
+  Bug#60646 - Fractions of seconds ignored in results
+*/
+DECLARE_TEST(t_bug60646)
+{
+  SQLCHAR buff[128];
+  TIMESTAMP_STRUCT ts;
+  SQLLEN len;
+  const char *expected= "2012-01-01 01:01:01.000001";
+
+  ok_sql(hstmt,
+        "SELECT timestamp('2012-01-01 01:01:01.000001')"            /*1*/
+        " ,timestamp('2012-01-01 01:01:01.100002')"                 /*2*/
+        " ,'2011-07-29 17:52:15.0000000009'"                        /*3*/
+        " ,'1000-01-01 12:00:00.000000001'"                         /*4*/
+        " ,time('2011-12-31 23:59:59.999999')"                      /*5*/
+        " ,ADDTIME('9999-12-31 23:59:59.999999', '1 1:1:1.000002')" /*6*/
+        ); 
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  /* Fields 1-4 checking conversions from date as a string
+  /* 1) just to be sure that everything is fine with string */
+  is_str(my_fetch_str(hstmt, buff, 1), expected, sizeof(expected));
+
+  /* 2) testing if fractional part is converted to nanoseconds correctly */
+  ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_TYPE_TIMESTAMP, &ts, sizeof(ts),
+                            &len));
+
+  is_num(ts.fraction, 100002000);
+
+  /* 3) fractional part is less than we care (less than nanosecond).
+        Test using string as MySQL does not support units less than a microsecond */
+  ok_stmt(hstmt, SQLGetData(hstmt, 3, SQL_C_TYPE_TIMESTAMP, &ts, sizeof(ts),
+                            &len));
+  is_num(ts.fraction, 0);
+
+  /* 4) testing if min fraction detected
+        Again - mysql supports microseconds only. thus using string
+   */
+  ok_stmt(hstmt, SQLGetData(hstmt, 4, SQL_C_TYPE_TIMESTAMP, &ts, sizeof(ts),
+                            &len));
+  is_num(ts.fraction, 1);
+
+  /* 5) if time is converted to timestamp - checking if current date is set
+        and if fractional part in place. former can actually fail if day is
+        being changed */
+
+  {
+    time_t sec_time= time(NULL);
+    struct tm * cur_tm;
+
+    ok_stmt(hstmt, SQLGetData(hstmt, 5, SQL_C_TYPE_TIMESTAMP, &ts, sizeof(ts),
+                              &len));
+    cur_tm= localtime(&sec_time);
+
+    is_num(ts.year, 1900 + cur_tm->tm_year);
+    is_num(ts.month, 1 + cur_tm->tm_mon);
+    is_num(ts.day, cur_tm->tm_mday);
+  }
+
+  is_num(ts.fraction, 999999000);
+
+  /* 6) Expecting an error because of longer date
+        At the moment ADDTIME('9999-12-31 23:59:59.999999', '1 1:1:1.000002')
+        will give you 10000-01-02 01:01:01.000001
+        Fixed in 5.6
+   */
+
+  if (!mysql_min_version(hdbc, "5.6.", 4))
+  {
+    expect_stmt(hstmt, SQLGetData(hstmt, 6, SQL_C_TYPE_TIMESTAMP, &ts, sizeof(ts),
+                              &len), SQL_ERROR);
+
+    if (check_sqlstate(hstmt, "22018") != OK)
+    {
+      return FAIL;
+    }
+  }
+
+  /* 5th col once again This time we get it in time struct. Thus we are
+     loosing fractioanl part. Thus the state has to be 01S07 and
+     SQL_SUCCESS_WITH_INFO returned */
+  {
+    SQL_TIME_STRUCT timestruct;
+
+    expect_stmt(hstmt, SQLGetData(hstmt, 5, SQL_C_TYPE_TIME, &timestruct,
+                            sizeof(timestruct), &len), SQL_SUCCESS_WITH_INFO);
+
+    if (check_sqlstate(hstmt, "01S07") != OK)
+    {
+      return FAIL;
+    }
+  }
+
+  expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+
+  return OK;
+}
+
+
+/* Bug#60648 ODBC prepared statements ignore fractional part of temporal data
+   types */
+DECLARE_TEST(t_bug60648)
+{
+  SQL_TIMESTAMP_STRUCT param, result;
+
+  param.year=     2011;
+  param.month=    8;
+  param.day=      6;
+  param.hour=     1;
+  param.minute=   2;
+  param.second=   3;
+  param.fraction= 1000;
+  ok_stmt(hstmt, SQLPrepare(hstmt, (SQLCHAR *)"select ?", SQL_NTS));
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_TYPE_TIMESTAMP,
+    SQL_TYPE_DATE, 0, 0, &param, 0, NULL));
+
+  expect_stmt(hstmt, SQLExecute(hstmt), SQL_ERROR);
+  is_num(check_sqlstate(hstmt, "22008"), OK);
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_TYPE_TIMESTAMP,
+    SQL_TYPE_TIME, 0, 0, &param, 0, NULL));
+
+  expect_stmt(hstmt, SQLExecute(hstmt), SQL_ERROR);
+  is_num(check_sqlstate(hstmt, "22008"), OK);
+
+  ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_TYPE_TIMESTAMP,
+    SQL_TYPE_TIMESTAMP, 0, 0, &param, 0, NULL));
+
+  ok_stmt(hstmt, SQLExecute(hstmt));
+
+  ok_stmt(hstmt, SQLFetch(hstmt));
+
+  ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_TYPE_TIMESTAMP, &result, 0,
+                            NULL));
+
+  is_num(1000, result.fraction);
+
+  return OK;
+}
+
+
+DECLARE_TEST(t_b13975271)
+{
+  if (!mysql_min_version(hdbc, "5.6.", 4))
+  {
+    skip("Necessary feature added in MySQL 5.6.*");
+  }
+  else
+  {
+    SQLCHAR ts[27];
+    SQLLEN len;
+
+    ok_sql(hstmt, "DROP TABLE IF EXISTS t_b13975271");
+    ok_sql(hstmt, "CREATE TABLE t_b13975271 (ts TIMESTAMP(6), dt DATETIME(6),\
+                    t TIME(6))");
+
+    strcpy((char *)ts, "2012-04-25 10:20:49.0194");
+
+    ok_stmt(hstmt, SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                    SQL_TIMESTAMP,0, 0, ts, sizeof(ts), NULL));
+    ok_stmt(hstmt, SQLBindParameter(hstmt, 2, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                    SQL_CHAR,0, 0, ts, sizeof(ts), NULL));
+    ok_stmt(hstmt, SQLBindParameter(hstmt, 3, SQL_PARAM_INPUT, SQL_C_CHAR,
+                                    SQL_CHAR,0, 0, ts, sizeof(ts), NULL));
+    ok_stmt(hstmt, SQLPrepare(hstmt, "INSERT INTO t_b13975271(ts,dt,t) \
+                                      VALUES (?,?,?)",
+                              SQL_NTS));
+    ok_stmt(hstmt, SQLExecute(hstmt));
+
+    ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
+
+    /* now fetch and verify the results .. */
+    ok_sql(hstmt, "SELECT * FROM t_b13975271");
+
+    ok_stmt(hstmt, SQLFetch(hstmt));
+    ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_C_CHAR, ts, sizeof(ts), &len));
+    is_str(ts, "2012-04-25 10:20:49.019400", 26);
+
+    /*To make sure that for next test we compare not the data from prev call */
+    ts[0]='\0';
+    ok_stmt(hstmt, SQLGetData(hstmt, 2, SQL_C_CHAR, ts, sizeof(ts), &len));
+    is_str(ts, "2012-04-25 10:20:49.0194", 24);
+    ok_stmt(hstmt, SQLGetData(hstmt, 3, SQL_C_CHAR, ts, sizeof(ts), &len));
+    is_str(ts, "10:20:49.0194", 13);
+
+    ok_stmt(hstmt, SQLFreeStmt(hstmt,SQL_CLOSE));
+
+    ok_sql(hstmt, "DROP TABLE IF EXISTS t_b13975271");
+  }
+
+  return OK;
+}
+
+
 BEGIN_TESTS
   ADD_TEST(my_ts)
   ADD_TEST(t_tstotime)
@@ -937,11 +1211,15 @@ BEGIN_TESTS
   ADD_TEST(t_bug12520)
   ADD_TEST(t_bug15773)
   ADD_TEST(t_bug9927)
-  ADD_TODO(t_bug30081)
+  ADD_TEST(t_bug30081)
   ADD_TEST(t_datecolumns)
   ADD_TEST(t_bug14414)
   ADD_TEST(t_bug30939)
   ADD_TEST(t_bug31009)
+  ADD_TEST(t_bug37342)
+  ADD_TEST(t_bug60646)
+  ADD_TEST(t_bug60648)
+  ADD_TEST(t_b13975271)
 END_TESTS
 
 
