@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -28,6 +28,9 @@
   A basic interface for writing tests that produces TAP-compliant output.
 */
 
+/* We don't want ansi calls to be mapped to unicode counterparts, but that does not work */
+/* #define SQL_NOUNICODEMAP 1*/
+
 #ifdef HAVE_CONFIG_H
 # include <myconf.h>
 #endif
@@ -55,7 +58,18 @@
 #include <time.h>
 
 
-void printMessage(char *fmt, ...) {
+/* Get routines for helping with Unicode conversion. */
+#define ODBCTAP
+
+typedef unsigned int UTF32;
+typedef unsigned short UTF16;
+typedef unsigned char UTF8;
+
+#include "../util/unicode_transcode.c"
+
+
+void printMessage(const char *fmt, ...)
+{
   va_list ap;
   va_start(ap, fmt);
   fprintf(stdout, "# ");
@@ -65,23 +79,57 @@ void printMessage(char *fmt, ...) {
 }
 
 
+/* Function that converts wchr_t string to char string for a debug output
+   Minimum guaranteed lenght of string it can convert is 2048/12=170. Stops if its
+   buffer cannot accomodate more characters from parameter string */
+const char * wstr4output(const wchar_t *wstr)
+{
+  size_t i;
+  static char str[2048];
+  char dbuf[16], *to= str;
+
+  for (i= 0; i < wcslen(wstr); ++i)
+  {
+    if (wstr[i] < 0x7f )
+    {
+      sprintf(dbuf, "%c", wstr[i]);
+    }
+    else
+    {
+      sprintf(dbuf, "[0x%x]", wstr[i]);
+    }
+
+    if (to - str + strlen(dbuf) + 1 < sizeof(str))
+    {
+      to+= sprintf(to, "%s", dbuf);
+    }
+    else
+    {
+      break;
+    }
+  }
+  
+  return str;
+}
+
 #define MAX_NAME_LEN 255
 #define MAX_COLUMNS 500
 #define MAX_ROW_DATA_LEN 1000
 #define MYSQL_NAME_LEN 64
+#define MAX_MEM_BLOCK_ELEMENTS 100
 
-SQLCHAR *mydriver= (SQLCHAR *)"{MySQL ODBC 3.51 Driver}";
+SQLCHAR *mydriver= (SQLCHAR *)"{MySQL ODBC 5.1 Driver}";
 SQLCHAR *mydsn= (SQLCHAR *)"test";
 SQLCHAR *myuid= (SQLCHAR *)"root";
 SQLCHAR *mypwd= (SQLCHAR *)"";
 SQLCHAR *mysock= NULL;
 int      myoption= 0, myport= 0;
+SQLCHAR *my_str_options= (SQLCHAR *)""; /* String for additional connection options */
 SQLCHAR *myserver= (SQLCHAR *)"localhost";
 SQLCHAR *mydb= (SQLCHAR *)"test";
 SQLCHAR *test_db= (SQLCHAR *)"client_odbc_test";
-
-int g_nCursor;
-char g_szHeader[501];
+/* Suffix is useful if a testsuite is run more than once */
+const SQLCHAR *testname_suffix="";
 
 #ifndef OK
 # define OK 0
@@ -125,6 +173,16 @@ typedef struct {
   int   expect;
 } my_test;
 
+
+typedef struct
+{
+  void *blk[MAX_MEM_BLOCK_ELEMENTS];
+  int  counter;
+} GCOLL;
+
+GCOLL gc_blk;
+
+
 #ifdef WIN32
 void test_timeout(int signum);
 HANDLE halarm= NULL;
@@ -138,12 +196,13 @@ DWORD WINAPI win32_alarm(LPVOID arg)
 #define ENABLE_ALARMS    int do_alarms= !getenv("DISABLE_TIMEOUT")
 #define RUN_TESTS_SIGNAL halarm= CreateEvent(NULL, FALSE, FALSE, NULL); \
                          if (do_alarms) \
-                           CreateThread(NULL, 0, win32_alarm, (LPVOID) 30, 0, NULL)
+                           CreateThread(NULL, 0, win32_alarm, (LPVOID) 60, 0, NULL); \
+                         do_alarms= 0
 #define RUN_TESTS_ALARM (void) SetEvent(halarm)
 #else
 #define ENABLE_ALARMS    int do_alarms= !getenv("DISABLE_TIMEOUT")
 #define RUN_TESTS_SIGNAL (void)signal(SIGALRM, test_timeout)
-#define RUN_TESTS_ALARM  if (do_alarms) alarm(30)
+#define RUN_TESTS_ALARM  if (do_alarms) alarm(60)
 #endif
 
 void mem_debug_init()
@@ -158,9 +217,24 @@ void mem_debug_init()
 #endif
 }
 
+/* 
+  Initialize garbage collector 
+  counter (or position)
+*/
+void mem_gc_init()
+{
+  gc_blk.counter= 0;
+}
+
+
 #define BEGIN_TESTS my_test tests[]= {
 #define ADD_TEST(name) { #name, name, OK   },
 #define ADD_TODO(name) { #name, name, FAIL },
+#ifdef EXPOSE_TOFIX_TESTS
+# define ADD_TOFIX(name) { #name, name, OK   },
+#else
+# define ADD_TOFIX(name) { #name, name, FAIL },
+#endif
 #define END_TESTS }; \
 void test_timeout(int signum __attribute__((unused))) \
 { \
@@ -177,6 +251,7 @@ int main(int argc, char **argv) \
   ENABLE_ALARMS; \
 \
   mem_debug_init(); \
+  mem_gc_init(); \
 \
   /* Set from environment, possibly overrided by command line */ \
   if (getenv("TEST_DSN")) \
@@ -204,7 +279,7 @@ int main(int argc, char **argv) \
 #define SET_DSN_OPTION(x) \
   myoption= (x);
 
-#define RUN_TESTS \
+#define RUN_TESTS_ONCE \
   setbuf(stdout, NULL); \
   num_tests= sizeof(tests) / sizeof(tests[0]); \
   printf("1..%d\n", num_tests); \
@@ -219,23 +294,27 @@ int main(int argc, char **argv) \
     int rc; \
     RUN_TESTS_ALARM; \
     rc= tests[i].func(hdbc, hstmt, henv); \
-    printf("%s %d - %s %s%s\n", \
+    printf("%s %d - %s%s %s%s\n", \
            (rc == OK || rc == SKIP) ? "ok" : "not ok", \
            i + 1, \
-           tests[i].name, \
+           tests[i].name, testname_suffix, \
            (tests[i].expect == FAIL ? "# TODO" : \
             rc == SKIP ? "# SKIP " : ""), \
            SKIP_REASON ? SKIP_REASON : ""); \
     if ((rc == FAIL) && (FAIL != tests[i].expect)) \
-      failcnt++; \
+      ++failcnt; \
     SKIP_REASON= NULL; /* Reset SKIP_REASON */ \
     /* Re-allocate statement to reset all its properties. */ \
     SQLFreeStmt(hstmt, SQL_DROP); \
     SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt); \
-  } \
-\
+    /* Freeing allocated memory */ \
+    mem_gc_flush(); \
+  }
+
+#define RUN_TESTS \
+  RUN_TESTS_ONCE \
   (void)free_basic_handles(&henv, &hdbc, &hstmt); \
-\
+  mem_gc_flush(); \
   exit(failcnt); \
 }
 
@@ -308,22 +387,24 @@ do { \
 
 
 /**
-  If calling a help function that doesn't return OK,
-  terminate as we would have using ok_stmt()
+  Verify that the results of an ODBC function call on a descriptor handle was
+  SQL_SUCCESS or SQL_SUCCESS_WITH_INFO.
 
-  @param call  The function call
+  @param desc Handle for descriptor object
+  @param call The function call
 */
-#define nok_pass_on(call) \
+#define ok_desc(desc, call) \
 do { \
-  int rc= (call); \
-  if (rc != OK) \
-    return rc; \
+  SQLRETURN rc= (call); \
+  print_diag(rc, SQL_HANDLE_DESC, (desc), #call, __FILE__, __LINE__); \
+  if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO) \
+    return FAIL; \
 } while (0)
 
 
 /**
   Verify that the result of an ODBC function call matches an expected
-  result, such as SQL_NO_DATA_FOUND.
+  result, such as SQL_ERROR or SQL_NO_DATA_FOUND.
 
   @param hnd       Handle for object
   @param type      Type of handle
@@ -431,6 +512,25 @@ do { \
 } while (0);
 
 
+/**
+  Verify that a wide string (wchar_t *) matches an expected value.
+
+  @param a     The string to compare
+  @param b     The string to compare against
+  @param c     The number of characters to compare
+*/
+#define is_wstr(a, b, c) \
+do { \
+  wchar_t *val_a= (a), *val_b= (b); \
+  int val_len= (int)(c); \
+  if (memcmp(val_a, val_b, val_len * sizeof(wchar_t)) != 0) { \
+    printf("# %s ('%*ls') != '%*ls' in %s on line %d\n", \
+           #a, val_len, val_a, val_len, val_b, __FILE__, __LINE__); \
+    return FAIL; \
+  } \
+} while (0);
+
+
 /**
   Verify that a number (long integer) matches an expected value.
 
@@ -439,15 +539,21 @@ do { \
 */
 #define is_num(a, b) \
 do { \
-  if (a != b) { \
+  long long a1= (a), a2= (b); \
+  if (a1 != a2) { \
     printf("# %s (%lld) != %lld in %s on line %d\n", \
-           #a, (long long)a, (long long)b, __FILE__, __LINE__); \
+           #a, a1, a2, __FILE__, __LINE__); \
     return FAIL; \
   } \
 } while (0);
 
 
-int check_sqlstate(SQLHSTMT hstmt, char *sqlstate)
+/* convenience macro for using statement */
+#define check_sqlstate(stmt, sqlstate) \
+  check_sqlstate_ex((stmt), SQL_HANDLE_STMT, (sqlstate))
+
+
+int check_sqlstate_ex(SQLHANDLE hnd, SQLSMALLINT hndtype, char *sqlstate)
 {
   SQLCHAR     sql_state[6];
   SQLINTEGER  err_code= 0;
@@ -455,7 +561,7 @@ int check_sqlstate(SQLHSTMT hstmt, char *sqlstate)
   SQLSMALLINT err_len= 0;
 
   memset(err_msg, 'C', SQL_MAX_MESSAGE_LENGTH);
-  SQLGetDiagRec(SQL_HANDLE_STMT, hstmt, 1, sql_state, &err_code, err_msg,
+  SQLGetDiagRec(hndtype, hnd, 1, sql_state, &err_code, err_msg,
                 SQL_MAX_MESSAGE_LENGTH - 1, &err_len);
 
   is_str(sql_state, (SQLCHAR *)sqlstate, 5);
@@ -469,7 +575,7 @@ int check_sqlstate(SQLHSTMT hstmt, char *sqlstate)
 static void print_diag(SQLRETURN rc, SQLSMALLINT htype, SQLHANDLE handle,
 		       const char *text, const char *file, int line)
 {
-  if (!SQL_SUCCEEDED(rc))
+  if (rc != SQL_SUCCESS)
   {
     SQLCHAR     sqlstate[6], message[SQL_MAX_MESSAGE_LENGTH];
     SQLINTEGER  native_error;
@@ -595,6 +701,10 @@ static int my_print_dashes(SQLHSTMT hstmt, SQLSMALLINT nCol)
             disp_size = col_len;
         if (disp_size < 4 && nullable)
             disp_size = 4;
+        /*
+          We cap disp_size to avoid problems when we have problems with
+          it being very large, such as 64-bit issues in the driver.
+        */
         disp_size= (disp_size > 512) ? 512 : disp_size;
         for (j=1; j < disp_size+3; j++)
           fprintf(stdout, "-");
@@ -627,6 +737,10 @@ static int my_print_data(SQLHSTMT hstmt, SQLUSMALLINT index,
         disp_size = col_len;
     if (disp_size < 4 && nullable)
         disp_size = 4;
+    /*
+      We cap disp_size to avoid problems when we have problems with
+      it being very large, such as 64-bit issues in the driver.
+    */
     disp_size= (disp_size > 512) ? 512 : disp_size;
     if (length == SQL_NULL_DATA)
         fprintf(stdout, "%-*s  |", (int)disp_size, "NULL");
@@ -660,14 +774,17 @@ int my_print_non_format_result(SQLHSTMT hstmt)
     SQLLEN      ind_strlen;
 
     rc = SQLNumResultCols(hstmt,&ncol);
+    
     mystmt_rows(hstmt,rc,-1);
 
-    for (nIndex = 1; nIndex <= ncol; nIndex++)
+    for (nIndex = 1; nIndex <= ncol; ++nIndex)
     {
         rc = SQLDescribeCol(hstmt,nIndex,szColName, MAX_NAME_LEN, NULL,
                             &pfSqlType,&pcColDef,&pcbScale,&pfNullable);
+        /* Returning in case of an error -nIndex we will see in the log column# */
         mystmt_rows(hstmt,rc,-nIndex);
 
+
         fprintf(stdout, "%s\t", szColName);
 
         rc = SQLBindCol(hstmt,nIndex, SQL_C_CHAR, szData[nIndex-1],
@@ -676,6 +793,7 @@ int my_print_non_format_result(SQLHSTMT hstmt)
     }
 
     fprintf(stdout,"\n");
+
     rc = SQLFetch(hstmt);
     while (SQL_SUCCEEDED(rc))
     {
@@ -704,7 +822,7 @@ SQLINTEGER myresult(SQLHSTMT hstmt)
     SQLUINTEGER nRowCount;
     SQLCHAR     ColName[MAX_NAME_LEN+1];
     SQLCHAR     Data[MAX_ROW_DATA_LEN+1];
-    SQLLEN      pcbLength, size;
+    SQLLEN      pcbLength;
     SQLUSMALLINT nIndex;
     SQLSMALLINT  ncol;
 
@@ -739,9 +857,6 @@ SQLINTEGER myresult(SQLHSTMT hstmt)
 
         for (nIndex=1; nIndex<= ncol; ++nIndex)
         {
-            rc = SQLColAttribute(hstmt,nIndex,SQL_DESC_DISPLAY_SIZE,NULL,0,
-                                 NULL,&size);
-            mystmt(hstmt,rc);
             rc = SQLGetData(hstmt, nIndex, SQL_C_CHAR, Data,
                             MAX_ROW_DATA_LEN,&pcbLength);
             mystmt(hstmt,rc);
@@ -771,7 +886,7 @@ SQLUINTEGER myrowcount(SQLHSTMT hstmt)
     rc = SQLFetch(hstmt);
     while (rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO)
     {
-        nRowCount++;
+        ++nRowCount;
         rc = SQLFetch(hstmt);
     }
     SQLFreeStmt(hstmt,SQL_UNBIND);
@@ -797,14 +912,22 @@ SQLRETURN tmysql_prepare(SQLHSTMT hstmt, char *sql_stmt)
 /**
   return integer data by fetching it
 */
-SQLINTEGER my_fetch_int(SQLHSTMT hstmt, SQLUSMALLINT irow)
+SQLINTEGER my_fetch_int(SQLHSTMT hstmt, SQLUSMALLINT icol)
 {
     SQLINTEGER nData;
     SQLLEN len;
 
-    SQLGetData(hstmt, irow, SQL_INTEGER, &nData, 0, &len);
-    printMessage("my_fetch_int: %ld (%ld)", (long int)nData, len);
-    return (len != SQL_NULL_DATA) ? nData : 0;
+    SQLGetData(hstmt, icol, SQL_INTEGER, &nData, 0, &len);
+    if (len == SQL_NULL_DATA)
+    {
+      printMessage("my_fetch_int: NULL");
+    }
+    else
+    {
+      printMessage("my_fetch_int: %ld (%ld)", (long int)nData, len);
+      return nData;
+    }
+    return 0;
 }
 
 
@@ -830,40 +953,69 @@ const char *my_fetch_str(SQLHSTMT hstmt, SQLCHAR *szData,SQLUSMALLINT icol)
     SQLLEN nLen;
 
     SQLGetData(hstmt,icol,SQL_CHAR,szData,MAX_ROW_DATA_LEN+1,&nLen);
-    printMessage(" my_fetch_str: %s(%ld)",szData,nLen);
+    /* If Null value - putting down smth meaningful. also that allows caller to
+       better/(in more easy way) test the value */
+    if (nLen < 0)
+    {
+      strcpy(szData, "(Null)"); 
+    }
+    printMessage(" my_fetch_str: %s(%ld)", szData, nLen);
     return((const char *)szData);
 }
 
 
-/*
-  Check if server running is MySQL
+/**
+  Convert a SQLWCHAR to wchar_t
 */
-int server_is_mysql(SQLHDBC hdbc)
+wchar_t *sqlwchar_to_wchar_t(SQLWCHAR *in)
 {
-    char driver_name[MYSQL_NAME_LEN];
+  static wchar_t buff[2048];
+  wchar_t *to= buff;
 
-    SQLGetInfo(hdbc,SQL_DRIVER_NAME,driver_name,MYSQL_NAME_LEN,NULL);
+  if (sizeof(wchar_t) == sizeof(SQLWCHAR))
+    return (wchar_t *)in;
 
-    if (strncmp(driver_name,"myodbc",6) >= 0 || strncmp(driver_name,"libmyodbc",9) >= 0)
-        return TRUE;
+  for ( ; *in && to < buff + sizeof(buff) - 2; in++)
+    to+= utf16toutf32((UTF16 *)in, (UTF32 *)to);
 
-    return FALSE;
+  *to= L'\0';
+
+  return buff;
 }
 
+
+/**
+  return wide string data, by fetching it and possibly converting it to wchar_t
+*/
+wchar_t *my_fetch_wstr(SQLHSTMT hstmt, SQLWCHAR *buffer, SQLUSMALLINT icol)
+{
+  SQLRETURN rc;
+  SQLLEN nLen;
+
+  rc= SQLGetData(hstmt, icol, SQL_WCHAR, buffer, MAX_ROW_DATA_LEN + 1, &nLen);
+  if (!SQL_SUCCEEDED(rc))
+    return L"";
+
+  buffer[nLen/sizeof(SQLWCHAR)]= 0;
+
+  return sqlwchar_to_wchar_t(buffer);
+}
+
+
 /*
   Check if driver supports SQLSetPos
 */
 int driver_supports_setpos(SQLHDBC hdbc)
 {
-    SQLRETURN    rc;
-    SQLUSMALLINT status= TRUE;
+  SQLRETURN    rc;
+  SQLUSMALLINT status= TRUE;
 
-    rc = SQLGetFunctions(hdbc, SQL_API_SQLSETPOS, &status);
-    mycon(hdbc,rc);
+  rc = SQLGetFunctions(hdbc, SQL_API_SQLSETPOS, &status);
+  mycon(hdbc,rc);
 
-    if (!status)
-        return FALSE;
-    return TRUE;
+  if (!status)
+    return FALSE;
+  return TRUE;
 }
 
 /*
@@ -871,19 +1023,27 @@ int driver_supports_setpos(SQLHDBC hdbc)
 */
 int mysql_min_version(SQLHDBC hdbc, char *min_version, unsigned int length)
 {
-    SQLCHAR server_version[MYSQL_NAME_LEN+1];
-    SQLRETURN rc;
+  SQLCHAR server_version[MYSQL_NAME_LEN+1];
+  SQLRETURN rc;
 
-    if (!server_is_mysql(hdbc))
-        return TRUE;
+  rc = SQLGetInfo(hdbc,SQL_DBMS_VER,server_version,MYSQL_NAME_LEN,NULL);
+  mycon(hdbc, rc);
 
-    rc = SQLGetInfo(hdbc,SQL_DBMS_VER,server_version,MYSQL_NAME_LEN,NULL);
-    mycon(hdbc, rc);
+  {
+    unsigned int major1= 0, major2= 0, minor1= 0, minor2= 0, build1= 0, build2= 0;
 
-    if (strncmp((char *)server_version, min_version, length) >= 0)
-        return TRUE;
+    sscanf(server_version, "%u.%u.%u", &major1, &minor1, &build1);
+    sscanf(min_version, "%u.%u.%u", &major2, &minor2, &build2);
 
-    return FALSE;
+    if ( major1 > major2 ||
+      major1 == major2 && (minor1 > minor2 ||
+                          minor1 ==  minor2 && build1 >= build2))
+    {
+		  return TRUE;
+    }
+  }
+
+  return FALSE;;
 }
 
 /*
@@ -891,15 +1051,15 @@ int mysql_min_version(SQLHDBC hdbc, char *min_version, unsigned int length)
 */
 int server_supports_trans(SQLHDBC hdbc)
 {
-    SQLSMALLINT trans;
-    SQLRETURN   rc;
+  SQLSMALLINT trans;
+  SQLRETURN   rc;
 
-    rc = SQLGetInfo(hdbc,SQL_TXN_CAPABLE,&trans,0,NULL);
-    mycon(hdbc,rc);
+  rc = SQLGetInfo(hdbc,SQL_TXN_CAPABLE,&trans,0,NULL);
+  mycon(hdbc,rc);
 
-    if (trans != SQL_TC_NONE)
-        return TRUE;
-    return FALSE;
+  if (trans != SQL_TC_NONE)
+      return TRUE;
+  return FALSE;
 }
 
 
@@ -930,20 +1090,26 @@ int mydrvconnect(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt, SQLCHAR *connIn)
 }
 
 
-int alloc_basic_handles(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt)
+/* Helper function for tests to get (additional) connection 
+   If dsn, uid, pwd or options is null - they defualt to mydsn, myuid, mypwd
+   and my_str_options, respectively.
+   myoption, mysock and myport values are used. */
+int get_connection(SQLHDBC *hdbc, const SQLCHAR *dsn, const SQLCHAR *uid,
+                   const SQLCHAR *pwd, const SQLCHAR *options)
 {
-  SQLCHAR   connIn[MAX_NAME_LEN+1], connOut[MAX_NAME_LEN+1];
+  SQLCHAR     connIn[MAX_NAME_LEN*2], connOut[MAX_NAME_LEN*2];
   SQLSMALLINT len;
+  SQLRETURN   rc;
 
-  ok_env(*henv, SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, henv));
+  /* Defaults */
+  if (dsn     == NULL) dsn=     mydsn;
+  if (uid     == NULL) uid=     myuid;
+  if (pwd     == NULL) pwd=     mypwd;
+  if (options == NULL) options= my_str_options;
 
-  ok_env(*henv, SQLSetEnvAttr(*henv, SQL_ATTR_ODBC_VERSION,
-                              (SQLPOINTER)SQL_OV_ODBC3, 0));
+  sprintf((char *)connIn, "DSN=%s;UID=%s;PWD=%s;DATABASE=test;OPTION=%d",
+          (char *)dsn, (char *)uid, (char *)pwd, myoption);
 
-  ok_env(*henv, SQLAllocHandle(SQL_HANDLE_DBC, *henv, hdbc));
-
-  sprintf((char *)connIn, "DSN=%s;USER=%s;PASSWORD=%s;DATABASE=test;OPTION=%d",
-          (char *)mydsn, (char *)myuid, (char *)mypwd, myoption);
   if (mysock && mysock[0])
   {
     strcat((char *)connIn, ";SOCKET=");
@@ -956,11 +1122,36 @@ int alloc_basic_handles(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt)
     strcat((char *)connIn, buff);
   }
 
-  ok_con(*hdbc, SQLDriverConnect(*hdbc, NULL, connIn, SQL_NTS, connOut,
-                                 MAX_NAME_LEN, &len, SQL_DRIVER_NOPROMPT));
+  if (options != NULL && options[0] > 0)
+  {
+    strcat(connIn, ";");
+    strcat(connIn, options);
+  }
 
-  ok_con(*hdbc, SQLSetConnectAttr(*hdbc, SQL_ATTR_AUTOCOMMIT,
-                                  (SQLPOINTER)SQL_AUTOCOMMIT_ON, 0));
+  rc= SQLDriverConnect(*hdbc, NULL, connIn, SQL_NTS, connOut,
+                                 MAX_NAME_LEN, &len, SQL_DRIVER_NOPROMPT);
+
+  if (SQL_SUCCEEDED(rc))
+  {
+    /* Let's neglect possibility that error returned by get_connection() can be
+       in fact error of turning autocommit on */
+    rc= SQLSetConnectAttr(*hdbc, SQL_ATTR_AUTOCOMMIT,
+                                  (SQLPOINTER)SQL_AUTOCOMMIT_ON, 0);
+  }
+
+  return rc;
+}
+
+int alloc_basic_handles(SQLHENV *henv, SQLHDBC *hdbc, SQLHSTMT *hstmt)
+{
+  ok_env(*henv, SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, henv));
+
+  ok_env(*henv, SQLSetEnvAttr(*henv, SQL_ATTR_ODBC_VERSION,
+                              (SQLPOINTER)SQL_OV_ODBC3, 0));
+
+  ok_env(*henv, SQLAllocHandle(SQL_HANDLE_DBC, *henv, hdbc));
+
+  ok_con(*hdbc, get_connection(hdbc, mydsn, myuid, mypwd, my_str_options));
 
   ok_con(*hdbc, SQLAllocHandle(SQL_HANDLE_STMT, *hdbc, hstmt));
 
@@ -985,6 +1176,106 @@ int free_basic_handles(SQLHENV *henv,SQLHDBC *hdbc, SQLHSTMT *hstmt)
 }
 
 
+/* 
+  Allocate size bytes and return a pointer 
+  to allocated memory with storing in 
+  garbage collector
+*/
+void *gc_alloc(size_t len)
+{
+  if (gc_blk.counter >= MAX_MEM_BLOCK_ELEMENTS || len <= 0)
+  {
+    printf("# GC Memory reached maximum limit counter: %d " \
+        "or length:%d less then equal to 0 in %s on line %d\n", 
+        gc_blk.counter, len, __FILE__, __LINE__); \
+    return NULL;
+  }
+
+  gc_blk.blk[gc_blk.counter]= malloc(len);
+  return gc_blk.blk[gc_blk.counter++];
+}
+
+
+/* 
+  Free allocated memory collected by
+  garbage collector
+*/
+int mem_gc_flush()
+{
+  int i= 0;
+  if (gc_blk.counter <= 0)
+  {
+    return FAIL;
+  }
+
+  while (i < gc_blk.counter)
+  {
+    free(gc_blk.blk[i]);
+    gc_blk.blk[i++]= 0;
+  }
+
+  return OK;
+}
+
+
+/**
+ Helper for possibly converting a (wchar_t *) to a (SQLWCHAR *).
+ TODO: [almost?] all uses of those macros in tests lead to memore leak,
+       as resulting pointer is not remembered and thus memory never freed
+*/
+#define W(string) dup_wchar_t_as_sqlwchar((string), \
+                                          sizeof(string) / sizeof(wchar_t))
+#define WL(string, len) dup_wchar_t_as_sqlwchar((string), (len))
+
+
+/**
+  Convert a wchar_t * to a SQLWCHAR * if a wchar_t is not the same as
+  SQLWCHAR. New space is allocated and never freed. Because this is used in
+  short-lived test programs, this is okay, if not ideal.
+*/
+SQLWCHAR *dup_wchar_t_as_sqlwchar(wchar_t *from, size_t len)
+{
+  if (sizeof(wchar_t) == sizeof(SQLWCHAR))
+  {
+    SQLWCHAR *to= gc_alloc(len * sizeof(SQLWCHAR));
+    memcpy(to, from, len * sizeof(wchar_t));
+    return to;
+  }
+  else
+  {
+    size_t i;
+    SQLWCHAR *to= gc_alloc(2 * len * sizeof(SQLWCHAR));
+    SQLWCHAR *out= to;
+    for (i= 0; i < len; i++)
+      to+= utf32toutf16((UTF32)from[i], (UTF16 *)to);
+    *to= 0;
+    return out;
+  }
+}
+
+
+/**
+ Helper for converting a (char *) to a (SQLWCHAR *)
+*/
+#define WC(string) dup_char_as_sqlwchar((string))
+
+
+/**
+  Convert a char * to a SQLWCHAR *. New space is allocated and never freed.
+  Because this is used in short-lived test programs, this is okay, if not
+  ideal.
+*/
+SQLWCHAR *dup_char_as_sqlwchar(SQLCHAR *from)
+{
+  SQLWCHAR *to= gc_alloc((strlen((char *)from) + 1) * sizeof(SQLWCHAR));
+  SQLWCHAR *out= to;
+  while (from && *from)
+    *(to++)= (SQLWCHAR)*(from++);
+  *to= 0;
+  return out;
+}
+
+
 /**
   Check if we are using a driver manager for testing.
 
