@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -29,8 +29,6 @@
 #ifndef _INSTALLER_H
 #define _INSTALLER_H
 
-/* Required for ulong and perhaps for something else too */
-#include "../MYODBC_MYSQL.h"
 #include "../MYODBC_CONF.h"
 #include "../MYODBC_ODBC.h"
 
@@ -106,7 +104,7 @@ typedef struct {
   SQLCHAR *sslcapath8;
   SQLCHAR *sslcipher8;
 
-  /* flags 1 */
+  /*  */
   BOOL return_matching_rows;
   BOOL allow_big_results;
   BOOL use_compressed_protocol;
@@ -115,7 +113,8 @@ typedef struct {
   BOOL auto_reconnect;
   BOOL auto_increment_null_search;
   BOOL handle_binary_as_char;
-  /* flags 2 */
+  BOOL can_handle_exp_pwd;
+  /*  */
   BOOL dont_prompt_upon_connect;
   BOOL dynamic_cursor;
   BOOL ignore_N_in_name_table;
@@ -123,7 +122,7 @@ typedef struct {
   BOOL dont_use_set_locale;
   BOOL pad_char_to_full_length;
   BOOL dont_cache_result;
-  /* flags 3 */
+  /*  */
   BOOL return_table_names_for_SqlDescribeCol;
   BOOL ignore_space_after_function_names;
   BOOL force_use_of_named_pipes;
@@ -142,8 +141,24 @@ typedef struct {
   BOOL no_information_schema;
   /* SSL */
   unsigned int sslverify;
+  unsigned int cursor_prefetch_number;
+  BOOL use_ssps;
 } DataSource;
 
+/* perhaps that is a good idea to have const ds object with defaults */
+extern const unsigned int default_cursor_prefetch;
+
+typedef struct{
+  SQLCHAR *type_name;
+  SQLINTEGER name_length;
+  SQLSMALLINT sql_type;
+  SQLSMALLINT mysql_type;
+  SQLUINTEGER type_length;
+  BOOL binary;
+}SQLTypeMap;
+
+#define TYPE_MAP_SIZE 32
+
 DataSource *ds_new();
 void ds_delete(DataSource *ds);
 int ds_set_strattr(SQLWCHAR **attr, const SQLWCHAR *val);
